# coding: utf-8

"""
    lastfm.py

    author: Adam Saponara <saponara TA gmail TOD com>
      desc: Sends your latest Last.fm track to the current buffer
     usage:
       /set plugins.var.python.lastfm.lastfm_username yourusername
       /lastfm
   license: GPLv3

   history:
       0.9 - 2022-01-25, Sébastien Helleu <flashcode@flashtux.org>
             Fix mixed spaces and tabs for indentation

       0.8 - 2020-03-18, prg <prg@xannode.com>
             Port to python3

       0.7 - 2016-01-29, timss <timsateroy@gmail.com>
             Fix UnicodeEncodeError

       0.6 - 2016-01-14, Lukas Martini <lutoma@ohai.su>
             Use Last.fm API as RSS feeds are broken

       0.5 - 2014-05-07, Kromonos <weechat@kromonos.net>
             fixed some simple bugs

       0.4 - 2011-11-21, Jimmy Zelinskie <jimmyzelinskie@gmail.com>
             changed default encoding to utf-8

       0.3 - 2011-03-11, Sébastien Helleu <flashcode@flashtux.org>
             get python 2.x binary for hook_process (fix problem when
             python 3.x is default python version)

       0.2 - using hook_process for last.fm call (prevents hang)
           - using ?limit=1 in last.fm call (faster, more efficient)

       0.1 - initial script

"""

import weechat
import requests

weechat.register("lastfm", "Adam Saponara", "0.9", "GPL3", "Sends your latest Last.fm track to the current buffer", "", "")

defaults = {
        "lastfm_username" : "yourusername",
        "command" : "/me is listening to %s"
}

cmd_hook_process = ""
cmd_buffer       = ""
cmd_stdout       = ""
cmd_stderr       = ""

for k, v in defaults.items():
        if not weechat.config_is_set_plugin(k):
                weechat.config_set_plugin(k, v)

def lastfm_cmd(data, buffer, args):
        global cmd_hook_process, cmd_buffer, cmd_stdout, cmd_stderr
        if cmd_hook_process != "":
                weechat.prnt(buffer, "Lastfm is already running!")
                return weechat.WEECHAT_RC_OK
        cmd_buffer = buffer
        cmd_stdout = ""
        cmd_stderr = ""
        python3_bin = weechat.info_get("python3_bin", "") or "python"
        cmd_hook_process = weechat.hook_process(
                python3_bin + " -c \"\n"
                "import sys, requests\n"
                "r = requests.get('https://ws.audioscrobbler.com/2.0/?method=user.getrecenttracks&user=%(username)s&api_key=618f9ef38b3d0fed172a88c45ae67f33&format=json&limit=1&extended=0')\n"
                "if not r.status_code == requests.codes.ok:\n"
                "    print >>sys.stderr, 'Could not fetch Last.fm RSS feed.',\n"
                "    exit()\n"
                "json = r.json()['recenttracks']['track'][0]\n"
                "print('{} – {}'.format(json['artist']['#text'], json['name'])),\n"
                "\"" % {"username" : weechat.config_get_plugin('lastfm_username')},
                10000, "lastfm_cb", "")
        return weechat.WEECHAT_RC_OK

def lastfm_cb(data, command, rc, stdout, stderr):
        global cmd_hook_process, cmd_buffer, cmd_stdout, cmd_stderr
        cmd_stdout += stdout
        cmd_stderr += stderr
        if int(rc) >= 0:
                if cmd_stderr != "":
                        weechat.prnt(cmd_buffer, "%s" % cmd_stderr)
                if cmd_stdout != "":
                        weechat.command(cmd_buffer, weechat.config_get_plugin("command") % cmd_stdout.replace('\n',''))
                cmd_hook_process = ""
        return weechat.WEECHAT_RC_OK

hook = weechat.hook_command(
        "lastfm",
        "Sends your latest Last.fm track to the current buffer. Before using /lastfm, set your Last.fm username like this:\n\n"
        "    /set plugins.var.python.lastfm.lastfm_username yourusername\n\n"
        "You can also customize the command that will be sent to the buffer like this:\n\n"
        "    /set plugins.var.python.lastfm.command Right now I'm listening to %s\n",
        "", "", "", "lastfm_cmd", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2010-2021 Sébastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Time-based scheduler, like cron and at.
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2021-11-07, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.6: replace calls to function hook_completion_list_add by
#                  completion_list_add
# 2021-05-02, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.5: add compatibility with WeeChat >= 3.2 (XDG directories)
# 2012-01-03, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.4: make script compatible with Python 3.x
# 2011-02-13, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.3: use new help format for command arguments
# 2010-07-31, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.2: add keyword "commands" to run many commands
# 2010-07-26, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
# 2010-07-20, Sébastien Helleu <flashcode@flashtux.org>:
#     script creation
#

SCRIPT_NAME    = "cron"
SCRIPT_AUTHOR  = "Sébastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "0.6"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Time-based scheduler, like cron and at"

import_ok = True

try:
    import weechat
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    import_ok = False

try:
    import os, stat, time
    from datetime import datetime, timedelta
except ImportError as message:
    print("Missing package(s) for %s: %s" % (SCRIPT_NAME, message))
    import_ok = False

# script options
cron_settings = {
    "auto_reload" : "on",          # auto reload cron file each minute if file has changed
                                   # if set to off, then command "/cron reload" must be executed manually
    "auto_save"   : "on",          # auto save jobs to file (when adding/removing job, and when unloading script)
    "filename"    : "%h/cron.txt", # cron filename
    "quiet_load"  : "off",         # silently load crontab file (no message displayed, except errors)
    "quiet_save"  : "on" ,         # silently save crontab file (no message displayed, except errors)
    "quiet_exec"  : "off",         # silently exec commands (no error displayed)
}

crontab = []
cron_last_read_time = 0
cron_commands = {
    "print"     : "print a message on buffer",
    "print_hl"  : "print a message on buffer with \"highlight\" notify on line",
    "print_msg" : "print a message on buffer with \"message\" notify on line",
    "command"   : "execute a command (starting with \"/\") or send text to buffer (like input)",
    "commands"  : "execute many commands separated by \";\"",
    "python"    : "evaluate python code",
}

# ================================[ cron jobs ]===============================

class AlwaysMatch(set):
    def __contains__(self, item):
        return True
    def __str__(self):
        return "*"

alwaysMatch = AlwaysMatch()

def cron_str2int(str):
    """ Convert day name to digit. """
    days = [ "sun", "mon", "tue", "wed", "thu", "fri", "sat" ]
    try:
        pos = days.index(str)
        if pos != ValueError:
            return pos
    except:
        try:
            value = int(str)
            return value
        except:
            return 0

def cron_str2set(str, min_value, max_value):
    """ Convert string with range to a set. """
    if str == "*":
        return alwaysMatch
    ret = set([])
    try:
        items = str.split(",")
        for item in items:
            values = item
            skip = 1
            pos = values.find("/")
            if pos > 0:
                try:
                    skip = int(values[pos+1:])
                except:
                    skip = 1
                values = values[0:pos]
            pos = values.find("-")
            if pos > 0:
                value1 = cron_str2int(values[0:pos])
                value2 = cron_str2int(values[pos+1:])
            else:
                if values == "*":
                    value1 = min_value
                    value2 = max_value
                else:
                    value1 = cron_str2int(values)
                    value2 = cron_str2int(values)
            ret = set.union(ret, set(range(value1, value2 + 1, skip)))
    except:
        weechat.prnt("", "%scron: error with time string \"%s\""
                     % (weechat.prefix("error"), str))
        return alwaysMatch
    return ret

def cron_set2str(obj):
    """ Convert a set to a string, for display (sort set). """
    if isinstance(obj, AlwaysMatch):
        return "*"
    l = list(obj)
    l.sort()
    return "%s" % l

class CronJob(object):
    """ Class for a job in crontab. """
    def __init__(self, minute=alwaysMatch, hour=alwaysMatch,
                 monthday=alwaysMatch, month=alwaysMatch, weekday=alwaysMatch,
                 repeat="*", buffer="core.weechat", command=""):
        self.minute = minute
        self.minutes = cron_str2set(minute, 0, 59)
        self.hour = hour
        self.hours = cron_str2set(hour, 0, 23)
        self.monthday = monthday
        self.monthdays = cron_str2set(monthday, 1, 31)
        self.month = month
        self.months = cron_str2set(month, 1, 12)
        self.weekday = weekday
        self.weekdays = cron_str2set(weekday, 0, 6)
        if repeat == "*":
            self.repeat = -1
        else:
            try:
                self.repeat = int(repeat)
                if self.repeat < 1:
                    self.repeat = 1
            except:
                self.remaining_exec = 1
        self.buffer = buffer
        self.command = command

    def str_repeat(self):
        if self.repeat < 0:
            return "*"
        return "%d" % self.repeat

    def __str__(self):
        """ Job to string. """
        return "%s %s %s %s %s %s %s %s" % (self.minute, self.hour, self.monthday,
                                            self.month, self.weekday,
                                            self.str_repeat(), self.buffer, self.command)

    def str_debug(self):
        """ Job to string with detail (to view sets). """
        return "%s %s %s %s %s %d %s %s" % (cron_set2str(self.minutes),
                                            cron_set2str(self.hours),
                                            cron_set2str(self.monthdays),
                                            cron_set2str(self.months),
                                            cron_set2str(self.weekdays),
                                            self.repeat, self.buffer, self.command)

    def matchtime(self, t):
        """ Check if this job matches given time (ie command must be executed). """
        return ((t.minute in self.minutes) and
                (t.hour in self.hours) and
                (t.day in self.monthdays) and
                (t.month in self.months) and
                ((t.isoweekday() % 7) in self.weekdays))

    def exec_command(self, userExec=False):
        """ Execute job command. """
        global cron_commands
        display_error = weechat.config_get_plugin("quiet_exec") == "off"
        buf = ""
        if self.buffer == "current":
            buf = weechat.current_buffer()
        else:
            items = self.buffer.split(".", 1)
            if len(items) >= 2:
                buf = weechat.buffer_search(items[0], items[1])
        if buf:
            argv = self.command.split(None, 1)
            if len(argv) > 0 and argv[0] in cron_commands:
                if argv[0] == "print":
                    weechat.prnt(buf, "%s" % argv[1])
                elif argv[0] == "print_hl":
                    weechat.prnt_date_tags(buf, 0, "notify_highlight", "%s" % argv[1])
                elif argv[0] == "print_msg":
                    weechat.prnt_date_tags(buf, 0, "notify_message", "%s" % argv[1])
                elif argv[0] == "command":
                    weechat.command(buf, "%s" % argv[1])
                elif argv[0] == "commands":
                    cmds = argv[1].split(";")
                    for cmd in cmds:
                        weechat.command(buf, "%s" % cmd)
                elif argv[0] == "python":
                    eval(argv[1])
            elif display_error:
                weechat.prnt("", "%scron: unknown command (\"%s\")"
                             % (weechat.prefix("error"), self.command))
        else:
            if display_error:
                weechat.prnt("", "%scron: buffer \"%s\" not found"
                             % (weechat.prefix("error"), self.buffer))
        if not userExec and self.repeat > 0:
            self.repeat -= 1

# ============================[ load/save crontab ]===========================

def cron_filename():
    """ Get crontab filename. """
    options = {
        'directory': 'config',
    }
    return weechat.string_eval_path_home(
        weechat.config_get_plugin("filename"), {}, {}, options)

def cron_str_job_count(number):
    """ Get string with "%d jobs". """
    if number <= 1:
        return "%d job" % number
    return "%d jobs" % number

def cron_load(force_message=False):
    """ Load crontab from file. """
    global crontab, cron_last_read_time
    display_message = weechat.config_get_plugin("quiet_load") == "off" or force_message
    crontab = []
    filename = cron_filename()
    if os.path.isfile(filename):
        f = open(filename)
        for line in f:
            line = line.lstrip().rstrip("\r\n")
            if not line.startswith("#"):
                argv = line.split(None, 7)
                if len(argv) >= 8:
                    crontab.append(CronJob(argv[0], argv[1], argv[2], argv[3],
                                           argv[4], argv[5], argv[6], argv[7]))
        f.close()
        cron_last_read_time = time.time()
        if display_message:
            weechat.prnt("", "cron: %s loaded from \"%s\""
                         % (cron_str_job_count(len(crontab)), filename))
    else:
        if cron_last_read_time != 0:
            cron_last_read_time = 0
            if display_message:
                weechat.prnt("", "cron: reset (file not found: \"%s\")" % filename)
        elif display_message:
            weechat.prnt("", "cron: file not found: \"%s\"" % filename)

def cron_save(force_message=False):
    """ Save crontab in file. """
    global crontab, cron_last_read_time
    display_message = weechat.config_get_plugin("quiet_save") == "off" or force_message
    filename = cron_filename()
    f = open(filename, "w")
    f.write("#\n")
    f.write("# WeeChat crontab for script cron.py\n")
    f.write("# format: min hour monthday month weekday repeat buffer command args\n")
    f.write("#\n")
    for job in crontab:
        f.write("%s\n" % job)
    f.close()
    cron_last_read_time = time.time()
    if display_message:
        weechat.prnt("", "cron: %s saved to \"%s\""
                     % (cron_str_job_count(len(crontab)), filename))

def cron_reload_needed():
    """ Return True if crontab must be reloaded, False if it is not needed. """
    global cron_last_read_time
    filename = cron_filename()
    if os.path.isfile(filename):
        return cron_last_read_time == 0 or os.stat(filename)[stat.ST_MTIME] > cron_last_read_time
    else:
        return cron_last_read_time != 0

# ============================[ crontab functions ]===========================

def cron_list(debug=False):
    """ Display list of jobs in crontab. """
    global crontab
    if len(crontab) == 0:
        weechat.prnt("", "cron: empty crontab")
    else:
        weechat.prnt("", "crontab:")
        for i, job in enumerate(crontab):
            if debug:
                str_job = "%s" % job.str_debug()
            else:
                str_job = "%s" % job
            weechat.prnt("", "  %s[%s%03d%s]%s %s"
                         % (weechat.color("chat_delimiters"), weechat.color("chat"),
                            i + 1,
                            weechat.color("chat_delimiters"), weechat.color("chat"),
                            str_job))

def cron_add(minute, hour, monthday, month, weekday, repeat, buffer, command):
    """ Add a job to crontab. """
    global crontab
    job = CronJob(minute, hour, monthday, month, weekday, repeat, buffer, command)
    crontab.append(job)
    weechat.prnt("", "cron: job added:  %s" % job)
    if weechat.config_get_plugin("auto_save") == "on":
        cron_save()

def cron_current_time():
    """ Get current time. """
    return datetime(*datetime.now().timetuple()[:5])

# ==============================[ at functions ]==============================

def cron_at_time(strtime):
    reftime = datetime.now()
    if strtime.startswith("+"):
        try:
            strtime = strtime[1:]
            delta = timedelta(minutes=1)
            unit = strtime[-1]
            if unit in ["m", "h", "d"]:
                value = int(strtime[:-1])
            else:
                value = int(strtime)
                unit = "m"
            if unit == "m":
                delta = timedelta(minutes=value)
            elif unit == "h":
                delta = timedelta(hours=value)
            elif unit == "d":
                delta = timedelta(days=value)
            reftime += delta
        except:
            return None
        return [reftime.hour, reftime.minute]
    else:
        items = strtime.split(":", 1)
        if len(items) >= 2:
            try:
                hour = int(items[0])
                minute = int(items[1])
            except:
                return None
            return [hour, minute]

# ================================[ commands ]================================

def cron_completion_time_cb(data, completion_item, buffer, completion):
    """ Complete with time, for command '/cron'. """
    weechat.completion_list_add(completion, "*",
                                0, weechat.WEECHAT_LIST_POS_BEGINNING)
    return weechat.WEECHAT_RC_OK

def cron_completion_repeat_cb(data, completion_item, buffer, completion):
    """ Complete with repeat, for command '/cron'. """
    weechat.completion_list_add(completion, "*",
                                0, weechat.WEECHAT_LIST_POS_BEGINNING)
    return weechat.WEECHAT_RC_OK

def cron_completion_buffer_cb(data, completion_item, buffer, completion):
    """ Complete with buffer, for command '/cron'. """
    infolist = weechat.infolist_get("buffer", "", "")
    while weechat.infolist_next(infolist):
        plugin_name = weechat.infolist_string(infolist, "plugin_name")
        name = weechat.infolist_string(infolist, "name")
        weechat.completion_list_add(completion,
                                    "%s.%s" % (plugin_name, name),
                                    0, weechat.WEECHAT_LIST_POS_SORT)
    weechat.infolist_free(infolist)
    weechat.completion_list_add(completion, "current",
                                0, weechat.WEECHAT_LIST_POS_BEGINNING)
    weechat.completion_list_add(completion, "core.weechat",
                                0, weechat.WEECHAT_LIST_POS_BEGINNING)
    return weechat.WEECHAT_RC_OK

def cron_completion_keyword_cb(data, completion_item, buffer, completion):
    """ Complete with cron keyword, for command '/cron'. """
    global cron_commands
    for command in sorted(cron_commands.keys()):
        weechat.completion_list_add(completion, command,
                                    0, weechat.WEECHAT_LIST_POS_END)
    return weechat.WEECHAT_RC_OK

def cron_completion_commands_cb(data, completion_item, buffer, completion):
    """ Complete with commands, for command '/cron'. """
    infolist = weechat.infolist_get("hook", "command", "")
    while weechat.infolist_next(infolist):
        command = weechat.infolist_string(infolist, "command")
        if command.startswith("/"):
            command = command[1:]
        if command:
            weechat.completion_list_add(completion, "/%s" % command,
                                        0, weechat.WEECHAT_LIST_POS_SORT)
    weechat.infolist_free(infolist)
    return weechat.WEECHAT_RC_OK

def cron_completion_number_cb(data, completion_item, buffer, completion):
    """ Complete with jobs numbers, for command '/cron'. """
    global crontab
    if len(crontab) > 0:
        for i in reversed(range(0, len(crontab))):
            weechat.completion_list_add(completion, "%d" % (i + 1),
                                        0, weechat.WEECHAT_LIST_POS_BEGINNING)
    return weechat.WEECHAT_RC_OK

def cron_completion_at_time_cb(data, completion_item, buffer, completion):
    """ Complete with time, for command '/at'. """
    weechat.completion_list_add(completion, "+5m",
                                0, weechat.WEECHAT_LIST_POS_END)
    weechat.completion_list_add(completion, "20:00",
                                0, weechat.WEECHAT_LIST_POS_END)
    return weechat.WEECHAT_RC_OK

def cron_cmd_cb(data, buffer, args):
    """ Command /cron. """
    global crontab, cron_commands
    if args in ["", "list"]:
        cron_list()
        return weechat.WEECHAT_RC_OK
    if args == "debug":
        cron_list(debug=True)
        return weechat.WEECHAT_RC_OK
    argv = args.split(None, 9)
    if len(argv) > 0:
        if argv[0] == "add":
            if len(argv) >= 10:
                if argv[8] not in cron_commands:
                    weechat.prnt("", "%scron: unknown keyword \"%s\""
                                 % (weechat.prefix("error"), argv[8]))
                    return weechat.WEECHAT_RC_ERROR
                cron_add(argv[1], argv[2], argv[3], argv[4], argv[5],
                         argv[6], argv[7], argv[8] + " " + argv[9])
                return weechat.WEECHAT_RC_OK
        elif argv[0] in ["del", "exec"]:
            if argv[0] == "del" and argv[1] == "-all":
                crontab = []
                weechat.prnt("", "cron: all jobs deleted")
                if weechat.config_get_plugin("auto_save") == "on":
                    cron_save()
                return weechat.WEECHAT_RC_OK
            try:
                number = int(argv[1])
                if number < 1 or number > len(crontab):
                    weechat.prnt("", "%scron: job number not found" % weechat.prefix("error"))
                    return weechat.WEECHAT_RC_OK
                if argv[0] == "del":
                    job = crontab.pop(number - 1)
                    weechat.prnt("", "cron: job #%d deleted (%s)" % (number, job))
                    if weechat.config_get_plugin("auto_save") == "on":
                        cron_save()
                elif argv[0] == "exec":
                    job = crontab[number - 1]
                    job.exec_command(userExec=True)
            except:
                weechat.prnt("", "%scron: job number not found" % weechat.prefix("error"))
            return weechat.WEECHAT_RC_OK
        elif argv[0] == "reload":
            cron_load(force_message=True)
            return weechat.WEECHAT_RC_OK
        elif argv[0] == "save":
            cron_save(force_message=True)
            return weechat.WEECHAT_RC_OK
    weechat.prnt("", "%scron: invalid arguments" % weechat.prefix("error"))
    return weechat.WEECHAT_RC_OK

def cron_at_cmd_cb(data, buffer, args):
    """ Command /at. """
    global crontab, cron_commands
    if args in ["", "list"]:
        cron_list()
        return weechat.WEECHAT_RC_OK
    if args == "debug":
        cron_list(debug=True)
        return weechat.WEECHAT_RC_OK
    argv = args.split(None, 3)
    if len(argv) >= 4:
        if argv[2] not in cron_commands:
            weechat.prnt("", "%scron: unknown keyword \"%s\""
                         % (weechat.prefix("error"), argv[2]))
            return weechat.WEECHAT_RC_ERROR
        hour_min = cron_at_time(argv[0])
        if hour_min == None:
            weechat.prnt("", "%scron: invalid time \"%s\""
                         % (weechat.prefix("error"), argv[0]))
            return weechat.WEECHAT_RC_OK
        cron_add(str(hour_min[1]), str(hour_min[0]), "*", "*", "*",
                 "1", argv[1], argv[2] + " " + argv[3])
        return weechat.WEECHAT_RC_OK
    weechat.prnt("", "%scron: invalid arguments" % weechat.prefix("error"))
    return weechat.WEECHAT_RC_OK

# ==================================[ timer ]=================================

def cron_timer_cb(data, remaining_calls):
    """ Timer called each minute. """
    global crontab
    t1 = cron_current_time()
    if weechat.config_get_plugin("auto_reload") == "on":
        if cron_reload_needed():
            cron_load()
    crontab2 = []
    for job in crontab:
        if job.matchtime(t1):
            job.exec_command()
        if job.repeat != 0:
            crontab2.append(job)
    crontab = crontab2
    return weechat.WEECHAT_RC_OK

# ==================================[ main ]==================================

def cron_unload():
    """ Called when script is unloaded. """
    if weechat.config_get_plugin("auto_save") == "on":
        cron_save()
    return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC,
                        "cron_unload", ""):
        # set default settings
        for option, default_value in cron_settings.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)
        # completions for commands
        weechat.hook_completion("cron_time", "cron time", "cron_completion_time_cb", "")
        weechat.hook_completion("cron_repeat", "cron repeat", "cron_completion_repeat_cb", "")
        weechat.hook_completion("cron_buffer", "cron buffer", "cron_completion_buffer_cb", "")
        weechat.hook_completion("cron_keyword", "cron keyword", "cron_completion_keyword_cb", "")
        weechat.hook_completion("cron_commands", "cron commands", "cron_completion_commands_cb", "")
        weechat.hook_completion("cron_number", "cron number", "cron_completion_number_cb", "")
        weechat.hook_completion("at_time", "at time", "cron_completion_at_time_cb", "")
        # commands /cron and /at
        str_buffer = "  buffer: buffer where command is executed\n" \
            "          (\"current\" for current buffer, \"core.weechat\" for WeeChat core buffer)\n"
        str_commands = " command: a keyword, followed by arguments:\n"
        for cmd in sorted(cron_commands.keys()):
            str_commands += "          - " + cmd + ": " + cron_commands[cmd] + "\n";
        weechat.hook_command("cron",
                             "Manage jobs in crontab",
                             "list || add <minute> <hour> <monthday> <month> <weekday> <repeat> <buffer> <command> || "
                             "del <number>|-all || exec <number> || reload|save",
                             "    list: display jobs in crontab\n"
                             "     add: add a job in crontab\n"
                             "  minute: minute (0-59)\n"
                             "    hour: hour (0-23)\n"
                             "monthday: day of month (1-31)\n"
                             "   month: month (1-12)\n"
                             " weekday: day of week (0-6, or name: "
                             "sun (0), mon (1), tue (2), wed (3), thu (4), fri (5), sat (6))\n"
                             "  repeat: number of times job will be executed, must be >= 1 or special value * (repeat forever)\n"
                             + str_buffer + str_commands +
                             "     del: remove job(s) from crontab\n"
                             "  number: job number\n"
                             "    -all: remove all jobs\n"
                             "    exec: execute a command for a job (useful for testing command)\n"
                             "  reload: reload crontab file (automatic by default)\n"
                             "    save: save current crontab to file (automatic by default)\n\n"
                             "Format for time and date is similar to crontab, see man 5 crontab.\n\n"
                             "Examples:\n"
                             "  Display \"short drink!\" at 12:00 each day:\n"
                             "    /cron add 0 12 * * * * core.weechat print short drink!\n"
                             "  Same example with python code:\n"
                             "    /cron add 0 12 * * * * core.weechat python weechat.prnt(\"\", \"short drink!\")\n"
                             "  Set away status on all servers at 23:30:\n"
                             "    /cron add 30 23 * * * * core.weechat command /away -all I am sleeping\n"
                             "  Remove away status on all servers at 07:00:\n"
                             "    /cron add 0 7 * * * * core.weechat command /away -all I am sleeping\n"
                             "  Set away status on all servers at 10:00 every sunday:\n"
                             "    /cron add 0 10 * * sun * core.weechat command /away -all I am playing tennis\n"
                             "  Say \"hello\" on IRC channel #private at 08:00 from monday to friday:\n"
                             "    /cron add 0 8 * * mon-fri * irc.freenode.#private command hello\n"
                             "  Display \"wake up!\" at 06:00 next monday, only one time, with highlight:\n"
                             "    /cron add 0 6 * * mon 1 core.weechat print_hl wake up!\n"
                             "  Delete first entry in crontab:\n"
                             "    /cron del 1",
                             "list"
                             " || add %(cron_time) %(cron_time) %(cron_time) "
                             "%(cron_time) %(cron_time) %(cron_repeat) %(cron_buffer) "
                             "%(cron_keyword) %(cron_commands)"
                             " || del %(cron_number)|-all"
                             " || exec %(cron_number)"
                             " || reload"
                             " || save",
                             "cron_cmd_cb", "")
        weechat.hook_command("at",
                             "Queue job for later execution",
                             "list || <time> <buffer> <command>",
                             "    list: display jobs in crontab\n"
                             "    time: time for job, can be absolute (HH:MM) or relative "
                             "with \"+\" followed by value and optional unit\n"
                             "          (units: m=minutes (default), h=hours, d=days)\n"
                             + str_buffer + str_commands + "\n"
                             "To manage scheduled jobs, or use date and time to schedule a job, "
                             "use /cron command.\n\n"
                             "Examples:\n"
                             "  Display \"water the garden\" at 20:00:\n"
                             "    /at 20:00 core.weechat print water the garden\n"
                             "  Display \"phone to mom\" in 15 minutes:\n"
                             "    /at +15 core.weechat print phone to mom\n"
                             "  Display \"phone to dad\" in 2 hours:\n"
                             "    /at +2h core.weechat print phone to dad",
                             "list|%(at_time) %(cron_buffer) %(cron_keyword) %(cron_commands)",
                             "cron_at_cmd_cb", "")
        # load crontab
        cron_load()
        # timer
        weechat.hook_timer(60 * 1000, 60, 0, "cron_timer_cb", "")



# # Weechat-Greentext
# Version:          v1
# Min WeeChat
#  version tested:  3.8
# Author:           AGVXOV
# Contact:          agvxov@gmail.com
# Project Home:     https://github.com/agvxov/weechat-greentext
# This script is Public Domain.
#
# Weechat script for applying imageboard formatting to messages. The following are supported:
#  + greentext
#  + purpletext
#  + redtext
# 
# Both inbound and outbound messages are colored.
# Since the coloring uses IRC color codes,
# outbound greentexting will be visible to both you and your friends.
#
import weechat
import re

SCRIPT_NAME    = "greentext"
SCRIPT_AUTHOR  = "AGVXOV"
SCRIPT_VERSION = "1"
SCRIPT_LICENSE = "PD"
SCRIPT_DESC    = "Colorize imageboard-style text formatting."

greentext_re  = re.compile("^\s*>.*$")
purpletext_re = re.compile("^\s*<.*$")
redtext_re    = re.compile("^.*(==.*==).*$")

COLOR_GREEN  = chr(3) + str(3)
COLOR_PURPLE = chr(3) + str(6)
COLOR_RED    = chr(3) + str(4) + chr(2)
COLOR_END    = chr(3) + str(0)

def hi_greentext(modifier, s):
	if greentext_re.search(s):
		if modifier == 'irc_out1_PRIVMSG':
			s = COLOR_GREEN + s
		else:
			s = weechat.color("green") + s
	return s

def hi_purpletext(modifier, s):
	if purpletext_re.search(s):
		if modifier == 'irc_out1_PRIVMSG':
			s = COLOR_PURPLE + s
		else:
			s = weechat.color("magenta") + s
	return s

def hi_redtext(modifier, s):
	if redtext_re.search(s):
		if modifier == 'irc_out1_PRIVMSG':
			m = redtext_re.search(s)
			s = s[:m.start(1)] + COLOR_RED + m.group(1) + COLOR_END + s[m.end(1):]
		else:
			s = weechat.color("red") + s
	return s

def hi(data, modifier, modifier_data, s):
	msg = weechat.info_get_hashtable('irc_message_parse', {'message': s})
	r = msg["text"]
	r = hi_greentext(modifier, r)
	r = hi_purpletext(modifier, r)
	r = hi_redtext(modifier, r)
	r = s[:-len(msg["text"])] + r
	return r
	
def main():
	if not weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
		return
	weechat.hook_modifier('irc_in2_privmsg', 'hi', '')
	weechat.hook_modifier('irc_out1_privmsg', 'hi', '')
	
if __name__ == "__main__":
	main()



# SPDX-FileCopyrightText: 2010 xt <xt@bash.no>
# SPDX-FileCopyrightText: 2025 ryoskzypu <ryoskzypu@proton.me>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# This script colors nicks in IRC channels in the actual message
# not just in the prefix section.
#
#
# Bugs:
#   https://github.com/ryoskzypu/weechat_scripts
#
# History:
# 2026-07-06: xt <xt@bash.no>
#   version 34: fix colorization of nicks containing regex-special characters
#               (e.g., hyphens) — re.escape() was applied before dictionary lookup,
#               causing escaped nick form (e.g., 'flash\-code') to not match unescaped
#               dict keys (e.g., 'flash-code'). Keep nick unescaped for lookup/display,
#               escape only when building the line-matching regex.
# 2025-05-08: ryoskzypu <ryoskzypu@proton.me>
#   version 33: add many improvements, features, and fixes
# 2023-10-30: Sébastien Helleu <flashcode@flashtux.org>
#   version 32: revert to info "nick_color" with WeeChat >= 4.1.1
# 2023-10-16: Sébastien Helleu <flashcode@flashtux.org>
#   version 31: use info "irc_nick_color" on IRC buffers with WeeChat >= 4.1.0
# 2022-11-07: mva
#   version 30: add ":" and "," to VALID_NICK regexp,
#               to don't reset colorization in input_line
# 2022-07-11: ncfavier
#   version 29: check nick for exclusion *after* stripping
#               decrease minimum min_nick_length to 1
# 2020-11-29: jess
#   version 28: fix ignore_tags having been broken by weechat 2.9 changes
# 2020-05-09: Sébastien Helleu <flashcode@flashtux.org>
#   version 27: add compatibility with new weechat_print modifier data
#               (WeeChat >= 2.9)
# 2018-04-06: Joey Pabalinas <joeypabalinas@gmail.com>
#   version 26: fix freezes with too many nicks in one line
# 2018-03-18: nils_2
#   version 25: fix unable to run function colorize_config_reload_cb()
# 2017-06-20: lbeziaud <louis.beziaud@ens-rennes.fr>
#   version 24: colorize utf8 nicks
# 2017-03-01, arza <arza@arza.us>
#   version 23: don't colorize nicklist group names
# 2016-05-01, Simmo Saan <simmo.saan@gmail.com>
#   version 22: invalidate cached colors on hash algorithm change
# 2015-07-28, xt
#   version 21: fix problems with nicks with commas in them
# 2015-04-19, xt
#   version 20: fix ignore of nicks in URLs
# 2015-04-18, xt
#   version 19: new option ignore nicks in URLs
# 2015-03-03, xt
#   version 18: iterate buffers looking for nicklists instead of servers
# 2015-02-23, holomorph
#   version 17: fix coloring in non-channel buffers (#58)
# 2014-09-17, holomorph
#   version 16: use weechat config facilities
#               clean unused, minor linting, some simplification
# 2014-05-05, holomorph
#   version 15: fix python2-specific re.search check
# 2013-01-29, nils_2
#   version 14: make script compatible with Python 3.x
# 2012-10-19, ldvx
#   version 13: Iterate over every word to prevent incorrect colorization of
#               nicks. Added option greedy_matching.
# 2012-04-28, ldvx
#   version 12: added ignore_tags to avoid colorizing nicks if tags are present
# 2012-01-14, nesthib
#   version 11: input_text_display hook and modifier to colorize nicks in input bar
# 2010-12-22, xt
#   version 10: hook config option for updating blacklist
# 2010-12-20, xt
#   version 0.9: hook new config option for weechat 0.3.4
# 2010-11-01, nils_2
#   version 0.8: hook_modifier() added to communicate with rainbow_text
# 2010-10-01, xt
#   version 0.7: changes to support non-irc-plugins
# 2010-07-29, xt
#   version 0.6: compile regexp as per patch from Chris quigybo@hotmail.com
# 2010-07-19, xt
#   version 0.5: fix bug with incorrect coloring of own nick
# 2010-06-02, xt
#   version 0.4: update to reflect API changes
# 2010-03-26, xt
#   version 0.3: fix error with exception
# 2010-03-24, xt
#   version 0.2: use ignore_channels when populating to increase performance.
# 2010-02-03, xt
#   version 0.1: initial (based on ruby script by dominikh)

import weechat
import re

# Debug data structures.
#from pprint import PrettyPrinter
#pp = PrettyPrinter(indent=4)

w = weechat

SCRIPT_NAME    = 'colorize_nicks'
SCRIPT_AUTHOR  = 'xt <xt@bash.no>'
SCRIPT_VERSION = '34'
SCRIPT_LICENSE = 'GPL'
SCRIPT_DESC    = 'Use the weechat nick colors in the chat area'

# Config file/options
config_file     = ''  # Pointer
config_option   = {}
ignore_channels = []  # ignore_channels
ignore_nicks    = []  # ignore_nicks

# Dict with every nick on every channel, with its color and prefix as lookup values.
colored_nicks = {}

# Regexes

colors_rgx = r'''
                 \031
                 (?:
                     \d{2}                      # Fixed 'weechat.color.chat.*' codes
                     |
                     (?:                        # Foreground
                         [F*]
                         [*!\/_%.|]?            # IRC colors (00–15)
                         \d{2}
                         |
                         (?: F@ | \*@)          # IRC colors (16–99) and WeeChat colors (16–255)
                         [*!\/_%.|]?
                         \d{5}
                     )
                     (?:                        # Background
                         ~
                         (?: \d{2} | @\d{5})
                     )?
                 )
             '''
attr_rgx   = r'''
                 (?: \032 | \033)
                 [\001-\006]
                 |
                 \031\034                       # Reset color and keep attributes
             '''
reset_rgx  = r'\034'
split_rgx  = rf'''
                 ({colors_rgx})                 # Colors
                 |
                 ({attr_rgx})                   # Attributes
                 |
                 ({reset_rgx})                  # Reset all
                 |
                                                # Chars
             '''
has_colors_rgx  = rf'{colors_rgx} | {attr_rgx}'
is_color_rgx    = rf'\A(?: {has_colors_rgx})\Z'
exact_color_rgx = rf'\A{colors_rgx}\Z'

# Dict of regexes to compile.
regex = {
        'colors':      colors_rgx,
        'attr':        attr_rgx,
        'reset':       reset_rgx,
        'split':       split_rgx,
        'has_colors':  has_colors_rgx,
        'is_color':    is_color_rgx,
        'exact_color': exact_color_rgx,
}

# Reset color code
reset = w.color('reset')

# Space hex code
space = '\x20'

# Unique escape codes
uniq_esc_nick = '\36'  # Nick
uniq_esc_pref = '\37'  # Prefix

def config_init():
    '''
    Initialization of configuration file.
    Sections: look.
    '''

    global config_file

    # Create config.
    if (config_file := w.config_new(SCRIPT_NAME, '', '')) == '':
        return 'failed to create config file'

    # Create 'look' section.
    if (section_look := w.config_new_section(
        config_file, 'look', 0, 0, '', '', '', '', '', '', '', '', '', '')) == '':
        w.config_free(config_file)
        return 'failed to create look section'

    # Create 'look' options.

    opts = [
            {
                'option':       'ignore_channels',
                'opt_type':     'string',
                'desc':         'comma separated list of channels to ignore',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      '',
                'value':        '',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'ignore_nicks',
                'opt_type':     'string',
                'desc':         'comma separated list of nicks to ignore',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      '',
                'value':        '',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'colorize_filter',
                'opt_type':     'boolean',
                'desc':         'colorize nicks in filtered messages from /filter',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      'off',
                'value':        'off',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'colorize_input',
                'opt_type':     'boolean',
                'desc':         'colorize nicks in input',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      'off',
                'value':        'off',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'irc_only',
                'opt_type':     'boolean',
                'desc':         'ignore non IRC messages; i.e. set buffer restrictions: plugin = irc, tags = irc_privmsg and irc_notice, type = channel and private',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      'off',
                'value':        'off',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'ignore_tags',
                'opt_type':     'string',
                'desc':         'comma separated list of tags to ignore; i.e. irc_join,irc_part,irc_quit',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      '',
                'value':        '',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'min_nick_length',
                'opt_type':     'integer',
                'desc':         'minimum length of nicks to colorize',
                'str_val':      '',
                'min_val':      1,
                'max_val':      20,
                'default':      '1',
                'value':        '1',
                'null_val':     0,
                'check_val_cb': '',
            },
            {
                'option':       'nick_suffixes',
                'opt_type':     'string',
                'desc':         'character set of nick suffixes; matches only one out of several characters',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      ':,',
                'value':        ':,',
                'null_val':     0,
                'check_val_cb': 'check_affix_cb',
            },
            # Default charset is based on IRC channel membership prefixes.
            {
                'option':       'nick_prefixes',
                'opt_type':     'string',
                'desc':         'character set of nick prefixes; matches only one out of several characters',
                'str_val':      '',
                'min_val':      0,
                'max_val':      0,
                'default':      '~&@%+',
                'value':        '~&@%+',
                'null_val':     0,
                'check_val_cb': 'check_affix_cb',
            }
    ]

    if (rc := set_options(config_file, section_look, opts)):
        return rc

def set_options(config, section, options):
    ''' Creates config file options of a section. '''

    for i in options:
        option = i['option']

        config_option[option] = w.config_new_option(
                config,
                section,
                option,
                i['opt_type'],
                i['desc'],
                i['str_val'],
                i['min_val'],
                i['max_val'],
                i['default'],
                i['value'],
                i['null_val'],
                i['check_val_cb'],
                '', '', '', '', '')

        if not config_option[option]:
            return f"failed to create config '{option}' option"

def config_read():
    ''' Reads the configuration file and updates config pointers. '''

    rc = w.config_read(config_file)

    try:
        if rc == w.WEECHAT_CONFIG_READ_MEMORY_ERROR:
            raise ValueError('not enough memory to read config file')
        elif rc == w.WEECHAT_CONFIG_READ_FILE_NOT_FOUND:
            raise ValueError('config file was not found')

    except ValueError as err:
        w.prnt('', f'{SCRIPT_NAME}\t{err.args[0]}')
        raise

def check_affix_cb(data, option, value):
    ''' Checks if affix option is empty. Note that it must have a value, and space
    (\x20) is ignored. '''

    if value == '':
        return 0

    return 1

def compile_regexes():
    ''' Compiles all script regexes for reuse. '''

    for k,v in regex.items():
        regex[k] = re.compile(v, flags=re.VERBOSE)

def debug_str(var, string):
    ''' Displays string information for debugging in core.weechat buffer. '''

    w.prnt('', f'{var}:')
    w.command('', f'/debug unicode {string}')
    w.prnt('', '')

def get_nick_color(buffer, nick, my_nick):
    ''' Retrieves nick color code from weechat. '''

    if nick == my_nick:
        return w.color(w.config_string(w.config_get('weechat.color.chat_nick_self')))
    else:
        version = int(w.info_get('version_number', '') or 0)

        # 'irc_nick_color' (deprecated since version 1.5, replaced by 'nick_color')
        if w.buffer_get_string(buffer, 'plugin') == 'irc' and version == 0x4010000:
            server = w.buffer_get_string(buffer, 'localvar_server')
            return w.info_get('irc_nick_color', f'{server},{nick}')

        return w.info_get('nick_color', nick)

def colorize_priv_nicks(buffer):
    ''' Colorizes nicks on IRC private buffers. '''

    # Reset the buffer dict to update nicks changes, since there is no nicklist
    # in private buffers.
    colored_nicks[buffer] = {}

    my_nick   = w.buffer_get_string(buffer, 'localvar_nick')
    priv_nick = w.buffer_get_string(buffer, 'localvar_channel')

    for nick in my_nick, priv_nick:
        nick_color = get_nick_color(buffer, nick, my_nick)

        colored_nicks[buffer][nick] = {
                'color':  nick_color,
                'prefix': '',
        }

def colorize_nicks(buffer, min_len, prefixes, suffixes, has_colors, line):
    ''' Finds every nick from the dict of colored nicks, in the line and colorizes
    them. '''

    chop_line            = line
    chop_match           = ''
    chop_match_after     = ''
    color_match          = ''
    colorized_nicks_line = ''
    nick_end             = reset

    # Mark the nick's end with a unique escape to identify its position on preserve_colors().
    if has_colors is not None:
        nick_end = uniq_esc_nick

    # Split words on spaces, since it is the most common word divider and is not
    # valid in 'nicks' on popular protocols like IRC and matrix; thus protocols
    # that allow spaces in 'nicks' are limited here.
    for word in re.split(f'{space}+', line.strip(f'{space}')):
        nick_prefix = ''  # Reset nick prefix.

        if word == '':
            continue

        # Get possible nick from word.
        nicks_rgx = rf'''
                        [{prefixes}]?     # Optional prefix char
                        (?P<nick> [^ ]+)
                     '''
        if (nick := re.search(nicks_rgx, word, flags=re.VERBOSE)) is not None:
            nick = nick.group('nick')

        # If the word is not a known nick and its last character is an option
        # suffix (e.g. colon ':' or comma ','), try to match the word without it.
        # This is necessary as 'foo:' is a valid nick, which could be addressed
        # as 'foo::'.
        if nick not in colored_nicks[buffer]:
            if re.search(rf'[{suffixes}]$', nick) is not None:
                nick = nick[:-1]

        # Nick exists on buffer.
        if nick in colored_nicks[buffer]:
            if nick in ignore_nicks or len(nick) < min_len:
                continue

            # Get its color.
            nick_color = colored_nicks[buffer][nick]['color']

            # Find nick in the line.
            nick_re = re.escape(nick)
            line_rgx = rf'''
                           (?: \A | [ ])             # Boundary
                           (?P<pref> [{prefixes}])?  # Optional prefix char
                           (?P<nick> {nick_re})
                           [{suffixes}]?             # "        suffix char
                           (?: \Z | [ ])             # Boundary
                       '''

            # Nick is found in the line.
            if (line_match := re.search(line_rgx, chop_line, flags=re.VERBOSE)) is not None:
                # In order to prevent the regex engine to needless find the nicks
                # at previous match positions, preserve the state by chopping the
                # line at the start and end positions of matches.

                # Start position of nick match.
                start = line_match.start('nick')

                # Get the real nick prefix from nicklist.
                if (pref_match := line_match.group('pref')) is not None:
                    nick_prefix = colored_nicks[buffer][nick]['prefix']

                    # If it exists, update the start position match.
                    if pref_match == w.string_remove_color(nick_prefix, ''):
                        start = line_match.start('pref')

                        # Mark the prefix with a unique escape to idenfity its
                        # position on preserve_colors().
                        if has_colors is not None:
                            nick_prefix = f'{uniq_esc_pref}{nick_prefix}'
                    else:
                        nick_prefix = ''

                # End position of nick match.
                end = line_match.end('nick')

                # Chop
                chop_till_match  = chop_line[:end]
                chop_after_match = chop_line[end:]

                # Concat the chopped strings while colorizing the nick, then update
                # the chopped line.
                nick_str     = f'{nick_prefix}{nick_color}{nick}{nick_end}'
                color_match += f'{chop_till_match[:start]}{nick_str}{chop_till_match[end:]}'
                chop_line    = chop_after_match

    if color_match:
        colorized_nicks_line = f'{color_match}{chop_after_match}'

    return colorized_nicks_line

def preserve_colors(line, colorized_nicks_line):
    '''
    If the line string is already colored, captures every color code before the nick
    match, for restoration after nick colorizing. Otherwise string colors after the
    nick are reset.

    Testing:
      1. Create an IRC channel.
           /j ##testing-weechat

      2. Create the nick 'nick111' with perlexec:
          /perlexec my $buffer = weechat::buffer_search('==', 'irc.libera.##testing-weechat'); my $group = weechat::nicklist_add_group($buffer, '', 'test_group', 'weechat.color.nicklist_group', 1); weechat::nicklist_add_nick($buffer, $group, 'nick111', 'blue', '@', 'lightgreen', 1)

      3. Send this message in the channel with script unloaded:
          /input insert \x0305<\x03043 \x02\x0307nick111 is awesome\x02 \x0314[0 user] \x0399\x1fhttps://github.com/ \x0305n\x0355i\x0384c\x0302k\x0f\x03921\x03091\x03381 /weechat/ https\x1f\x16:// nick111 .org/

      4. Repeat step 3 with the script loaded. It should colorize the nicks and
         preserve all colors.
         The string is inspired by ##hntop messages and modified to cover some corner cases.
    '''

    new_line      = ''
    split_line    = []
    split_line_nc = []
    color_codes   = ''
    idx           = 0
    match         = 0

    # Split all color codes and bytes from the lines.
    split_line    = [x for x in regex['split'].split(line)                 if x is not None and x]
    split_line_nc = [y for y in regex['split'].split(colorized_nicks_line) if y is not None and y]

    # Debug split lists.
    #w.prnt('', f'split_line:'    + pp.pformat(split_line))
    #w.prnt('', f'split_line_nc:' + pp.pformat(split_line_nc))

    # Iterate through the original split list, comparing every char against the
    # uncolored list; while reconstructing the new line with saved color codes.
    for i in split_line:
        #w.prnt('', f'i: ' + pp.pformat(f'{i}'))
        #w.prnt('', f"split_line_nc[{idx}]: " + pp.pformat(f'{split_line_nc[idx]}'))

        # It is a color code, so append its codes to be restored.
        if regex['is_color'].search(i) is not None:
            color_codes += i
            #w.prnt('', f'color_codes: ' + pp.pformat(f'{color_codes}'))

            # Append the codes if not inside a nick match.
            if not match:
                new_line += i

            continue
        # Remove saved codes if a reset code is found.
        elif i == reset:
            if not match:
                new_line += i

            color_codes = ''
            continue
        elif split_line_nc[idx]:
            # It is a char, so compare it against the uncolored's char.
            if i == split_line_nc[idx]:
                new_line += i
                idx      += 1

                continue
            # If the char is in a nick match and uncolored's is a unique nick
            # escape code, restore the saved codes, then advance the index.
            elif match and split_line_nc[idx] == uniq_esc_nick:
                #w.prnt('', f"split_line_nc[{idx} + 1]: " + pp.pformat(f'{split_line_nc[idx + 1]}'))

                # If the chars match, advance the index.
                if split_line_nc[idx + 1] == i:
                    new_line += f'{reset}{color_codes}{i}'
                    idx      += 2
                    match     = 0

                    continue
            # It is a unique prefix escape code, so get its color code and char,
            # then advance uncolored's index to the start of colorized nick match.
            elif split_line_nc[idx] == uniq_esc_pref:
                prefix    = f'{split_line_nc[idx + 1]}{i}'
                new_line += prefix
                idx      += 3

                continue
            # It is the start of a colorized nick match, so colorize the new line,
            # then advance uncolored's index to the current char.
            elif (split_match := regex['exact_color'].search(split_line_nc[idx])) is not None:
                #w.prnt('', f"split_line_nc[{idx} + 1]: " + pp.pformat(f'{split_line_nc[idx + 1]}'))

                nick_color  = split_match.group(0)
                idx        += 1
                new_line   += f'{reset}{nick_color}{split_line_nc[idx]}'
                match       = 1

                # If the chars match, advance the index.
                if i == split_line_nc[idx]:
                    idx += 1
                    continue

    return new_line

def init_colorize(buffer, message):
    ''' Initializes the process of nicks colorizing. '''

    colorized_nicks_msg = ''
    new_msg             = ''

    # Get options.
    min_len      = w.config_integer(config_option['min_nick_length'])
    pref_charset = re.escape(w.config_string(config_option['nick_prefixes']))
    suff_charset = re.escape(w.config_string(config_option['nick_suffixes']))

    # Check if message has color codes.
    has_colors = regex['has_colors'].search(message)

    # Remove any color codes from message in order to match and colorize the strings correctly.
    msg_nocolor = w.string_remove_color(message, '')

    # Find and colorize the nicks.
    colorized_nicks_msg = colorize_nicks(buffer, min_len, pref_charset, suff_charset, has_colors, msg_nocolor)

    # Preserve colors from message.
    if has_colors is not None and colorized_nicks_msg:
        new_msg = preserve_colors(message, colorized_nicks_msg)

    # Debug the message string.
    #debug_str('message', message)

    # Update the message.

    if colorized_nicks_msg:
        #debug_str('colorized_nicks_msg', colorized_nicks_msg)
        message = colorized_nicks_msg

    if new_msg:
        #debug_str('new_msg', new_msg)
        message = new_msg

    return message

def colorize_cb(data, hashtable):
    '''
    Callback that does the colorizing of nicks from messages and returns a new message.

    Testing:
      1. Create an IRC channel:
         /j ##testing-weechat

      2. Create the nicks: alice, :alicee, alicee:, :alicee:, and utf8©nick
         with perlexec:
           /perlexec my @nicks = qw(alice :alicee alicee: :alicee: utf8©nick); my $buffer = weechat::buffer_search('==', 'irc.libera.##testing-weechat'); my $group = weechat::nicklist_add_group($buffer, '', 'test_group', 'weechat.color.nicklist_group', 1); foreach my $i (@nicks) { weechat::nicklist_add_nick($buffer, $group, $i, 'default', '@', 'lightgreen', 1) }

      3. Then paste and send this string:
         hey alicee and utf8©nickz, how are you? sorry, alice and utf8©nick   @alicee: @:alicee @:alicee: aaaliceee @:alicee:: @::alicee:: @alicee:: %alicee:, ~:alicee,  Nice to meet you @:alicee,,  &:alicee:, @:alicee,: :alicee, :alicee alicee: <3 :alicee: :alicee::: +utf8©nick: :-) bye

      4. The colors that matter are in the message, so ignore the static nicklist
         colors. The nicks in message should be colorized correctly based on weechat's
         color algorithm, and respect the script affixes.
         Insert a reverse color code (^Cv) at the beggining of string, if having
         trouble on seeing the colors.
    '''

    buffer    = hashtable['buffer']
    tags      = hashtable['tags'].split(',')
    displayed = hashtable['displayed']
    message   = hashtable['message']

    plugin  = w.buffer_get_string(buffer, 'localvar_plugin')
    bufname = w.buffer_get_string(buffer, 'localvar_name')
    buftype = w.buffer_get_string(buffer, 'localvar_type')
    channel = w.buffer_get_string(buffer, 'localvar_channel')

    irc_only = w.config_boolean(config_option['irc_only'])

    # Colorize only IRC user messages.
    if plugin == 'irc' or irc_only and plugin != 'irc':
        # There is no point in colorizing non channel/private buffers, and IRC
        # tags other than 'irc_privmsg/notice', since tags i.e. irc_join/part/quit
        # are already colored.
        if buftype != 'channel' and buftype != 'private' or tags[0] != 'irc_privmsg' and tags[0] != 'irc_notice':
            return hashtable

    # Colorize nicks on IRC private buffers.
    if plugin == 'irc' and buftype == 'private':
        colorize_priv_nicks(buffer)

    # Check if buffer has colorized nicks.
    if not colored_nicks.get(buffer):
        return hashtable

    # Check if channel is ignored.
    if channel and channel in ignore_channels:
        return hashtable

    # Do not colorize if an ignored tag is present in message.
    tag_ignores = w.config_string(config_option['ignore_tags']).split(',')
    for tag in tags:
        if tag in tag_ignores:
            return hashtable

    # Do not colorize if message is filtered.
    if displayed == '0' and not w.config_boolean(config_option['colorize_filter']):
        return hashtable

    # Init colorizing process.
    message = init_colorize(buffer, message)

    # Debug the hashtable.
    #w.prnt('', 'hashtable:\n' + pp.pformat(hashtable))

    # Update the hashtable.
    hashtable['message'] = message

    return hashtable

def colorize_input_cb(data, modifier, modifier_data, line):
    ''' Callback that does the colorizing of nicks from weechat's input. '''

    if not w.config_boolean(config_option['colorize_input']):
        return line

    buffer  = w.current_buffer()
    plugin  = w.buffer_get_string(buffer, 'localvar_plugin')
    buftype = w.buffer_get_string(buffer, 'localvar_type')
    channel = w.buffer_get_string(buffer, 'localvar_channel')

    irc_only = w.config_boolean(config_option['irc_only'])

    # Colorize only IRC user messages.
    if plugin == 'irc' or irc_only and plugin != 'irc':
        # There is no point in colorizing non channel/private buffers.
        if buftype != 'channel' and buftype != 'private':
            return line

    # Check if current buffer has colorized nicks.
    if not colored_nicks.get(buffer):
        return line

    # Check if current channel is ignored.
    if channel and channel in ignore_channels:
        return line

    # Decode IRC colors from input.
    if plugin == 'irc':
        line = w.hook_modifier_exec('irc_color_decode', '1', line)

    # Init colorizing process.
    line = init_colorize(buffer, line)

    return line

def populate_nicks_cb(*args):
    ''' Callback that fills the colored nicks dict with all nicks weechat can see,
    and what color and prefix it has assigned to it. '''

    bufname      = ''
    prefix_color = ''
    nick_prefix  = ''
    irc_only     = w.config_boolean(config_option['irc_only'])

    # Get nicks only in IRC buffers.
    if irc_only:
        bufname = 'irc.*'

    # Get list of buffers.
    if not (buffers := w.infolist_get('buffer', '', bufname)):
        w.prnt('', f'{SCRIPT_NAME}\tfailed to get list of buffers')
        return w.WEECHAT_RC_ERROR

    while w.infolist_next(buffers):
        buffer_ptr = w.infolist_pointer(buffers, 'pointer')
        channel    = w.buffer_get_string(buffer_ptr, 'localvar_channel')

        # Skip non-IRC channel buffers.
        if irc_only and not w.info_get('irc_is_channel', channel):
            continue

        my_nick = w.buffer_get_string(buffer_ptr, 'localvar_nick')

        if (nicklist := w.infolist_get('nicklist', buffer_ptr, '')):
            while w.infolist_next(nicklist):
                if buffer_ptr not in colored_nicks:
                    colored_nicks[buffer_ptr] = {}

                # Skip nick groups.
                if w.infolist_string(nicklist, 'type') != 'nick':
                    continue

                # Get nicks colors.
                nick       = w.infolist_string(nicklist, 'name')
                nick_color = get_nick_color(buffer_ptr, nick, my_nick)

                # Get nicks prefixes.
                prefix = w.infolist_string(nicklist, 'prefix')
                if prefix != space:
                    prefix_color = w.color(w.infolist_string(nicklist, 'prefix_color'))
                    nick_prefix  = f'{prefix_color}{prefix}'

                # Populate
                colored_nicks[buffer_ptr][nick] = {
                        'color':  nick_color,
                        'prefix': nick_prefix,
                }
                nick_prefix = ''

        w.infolist_free(nicklist)

    w.infolist_free(buffers)

    #w.prnt('', 'colored_nicks:\n' + pp.pformat(colored_nicks))

    return w.WEECHAT_RC_OK

def add_nick_cb(data, signal, signal_data):
    ''' Callback that adds a nick to the dict of colored nicks, when a nick is
    added to the nicklist. '''

    # Nicks can have ',' in them in some protocols.
    buffer, nick = signal_data.split(',', maxsplit=1)

    if buffer not in colored_nicks:
        colored_nicks[buffer] = {}

    # Get nick color.
    my_nick    = w.buffer_get_string(buffer, 'localvar_nick')
    nick_color = get_nick_color(buffer, nick, my_nick)

    # Get nick prefix.
    nick_prefix = ''
    if (nicklist := w.infolist_get('nicklist', buffer, f'nick_{nick}')):
        while w.infolist_next(nicklist):
            prefix = w.infolist_string(nicklist, 'prefix')

            if prefix != space:
                prefix_color = w.color(w.infolist_string(nicklist, 'prefix_color'))
                nick_prefix  = f'{prefix_color}{prefix}'

    # Update
    colored_nicks[buffer][nick] = {
            'color':  nick_color,
            'prefix': nick_prefix,
    }

    w.infolist_free(nicklist)
    w.infolist_free(buffer)

    #w.prnt('', 'colored_nicks:\n' + pp.pformat(colored_nicks))

    return w.WEECHAT_RC_OK

def remove_nick_cb(data, signal, signal_data):
    ''' Callback that removes a nick from the dict of colored nicks, when a nick is
    removed from the nicklist. '''

    # Nicks can have ',' in them in some protocols.
    buffer, nick = signal_data.split(',', maxsplit=1)

    if buffer in colored_nicks and nick in colored_nicks[buffer]:
        del colored_nicks[buffer][nick]

    #w.prnt('', 'colored_nicks:\n' + pp.pformat(colored_nicks))

    return w.WEECHAT_RC_OK

def remove_priv_buffer_cb(data, signal, buffer):
    ''' Callback that removes an IRC private buffer from the dict of colored nicks,
    when the buffer is closing. '''

    # For some reason, weechat crashes if the hook signal is set to 'buffer_closed'
    # while trying to get the 'localvar_*' strings.
    # Perhaps the buffer pointer is not valid anymore because it was closed?
    plugin  = w.buffer_get_string(buffer, 'localvar_plugin')
    buftype = w.buffer_get_string(buffer, 'localvar_type')

    if plugin == 'irc' and buftype == 'private' and buffer in colored_nicks:
        del colored_nicks[buffer]

    #w.prnt('', 'colored_nicks:\n' + pp.pformat(colored_nicks))

    return w.WEECHAT_RC_OK

def update_blacklist_cb(*args):
    ''' Callback that sets the blacklist for channels and nicks. '''

    global ignore_channels, ignore_nicks

    ignore_channels = w.config_string(config_option['ignore_channels']).split(',')
    ignore_nicks    = w.config_string(config_option['ignore_nicks']).split(',')

    return w.WEECHAT_RC_OK

if __name__ == '__main__':
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        # Initialize config options and regexes.
        try:
            if (msg := config_init()):
                raise ValueError(msg)

        except ValueError as err:
            w.prnt('', f'{SCRIPT_NAME}\t{err.args[0]}')
            raise

        config_read()
        compile_regexes()

        # Run once to get data ready.
        update_blacklist_cb()
        populate_nicks_cb()

        # Hooks

        # Colorize nicks.
        w.hook_line('', '', '', 'colorize_cb', '')                          # Message
        w.hook_modifier('250|input_text_display', 'colorize_input_cb', '')  # Input

        # Update nicks.
        w.hook_signal('nicklist_nick_added', 'add_nick_cb', '')
        w.hook_signal('nicklist_nick_removed', 'remove_nick_cb', '')
        w.hook_signal('buffer_closing', 'remove_priv_buffer_cb', '')

        # Repopulate nicks on colors changes from weechat's options.
        w.hook_config('weechat.color.chat_nick_colors', 'populate_nicks_cb', '')
        w.hook_config('weechat.look.nick_color_hash', 'populate_nicks_cb', '')
        w.hook_config('irc.color.nick_prefixes', 'populate_nicks_cb', '')

        # Update blacklists.
        w.hook_config(f'{SCRIPT_NAME}.look.ignore_*', 'update_blacklist_cb', '')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by drubin <drubin at smartcube.co.za>
# Copyright (c) 2017 Sébastien Helleu <flashcode at flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Allows you to visually see if there are updates to your weechat system.

# Versions:
#
# 0.7 flashcode - fix error when next release is scheduled today
# 0.6 flashcode - add compatibility with WeeChat >= 3.2 (XDG directories)
# 0.5 flashcode - fix URL with infos, fix download of infos, fix PEP8 errors
#                 and refactor code
# 0.4 drubin    - changed default weechat hostname
# 0.3 nils_2    - third release.
#               - *fixed bug* every time the item_bar was updated, script did a
#                 read access to homepage
#               - get python 2.x binary for hook_process (fix problem when
#                 python 3.x is default python
#                 version, requires WeeChat >= 0.3.4)
#               - new option "git pull". executes "git pull" if "true" and a
#                 new git version is available
# 0.2 nils_2    - second release.
#               - countdown to next stable release added.
#               - now using hook_signal(day_changed) instead of hook_timer()
#               └-> option "update_interval" is obsolet now.
#               - hook_config() added.
#               - type missmatched removed if git_compile_location wasn't set
# 0.1 drubin   - First release.
#              - Basic functionality with url getting and compairing version.

SCRIPT_NAME = "update_notifier"
SCRIPT_AUTHOR = "drubin <drubin at smartcube.co.za>"
SCRIPT_VERSION = "0.7"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Notifiers users of updates to weechat."

import_ok = True
import os
from time import *
try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    import_ok = False


# script options
settings = {
    "uses_git": "false",
    "uses_devel": "false",
    "git_pull": "false",
    "git_compile_location": "",
    "update_text": "New devel version available",
    "update_text_stable": "New stable version %s available",
    "start_counting": "30",
    "start_countdown": "10",
    "next_stable_text": "%d day(s) left to version %s",
    "color_default": "default",
    "color_countdown": "red",
}

# Not a config because it should not change ever
URL_INFOS = "https://weechat.org/dev/info/all/"
INFOS_FILENAME = "infos.txt"
BAR_ITEM_NAME = "updnotf"

# How long to wait for download
TIMEOUT_DOWNLOAD = 60 * 1000


def un_cache_dir():
    options = {
        'directory': 'cache',
    }
    filename = weechat.string_eval_path_home('%%h/%s' % SCRIPT_NAME,
                                             {}, {}, options)
    if not os.path.isdir(filename):
        os.makedirs(filename, mode=0o700)
    return filename


def get_version(as_number=False):
    """Gets the version number of weechat, both number and string."""
    if as_number:
        return weechat.info_get("version_number", "")
    else:
        return weechat.info_get("version", "")


def get_filename_infos():
    """Return the path to the file with infos."""
    return os.path.join(un_cache_dir(), INFOS_FILENAME)


def get_cur_git_version():
    path = weechat.config_get_plugin("git_compile_location")
    if path == "":
        return None

    f = os.popen("cd %s && git rev-parse HEAD" % path)
    stuff = f.readline()
    f.close()
    return stuff.strip()


def do_git_pull():
    if (weechat.config_get_plugin("git_pull") == "false"):
        return
    path = weechat.config_get_plugin("git_compile_location")
    if path != "":
        f = os.popen("cd %s && git pull 2>&1" % path)
        stuff = f.readline()
        weechat.prnt("", weechat.prefix("action") +
                     weechat.color(weechat.config_color(
                         weechat.config_get("weechat.color.chat_nick_self"))) +
                     SCRIPT_NAME + ":")
        weechat.prnt("", stuff)
        f.close()


def get_info_ver(info):
    with open(get_filename_infos(), "r") as f:
        for line in f.readlines():
            items = line.strip().split(":", 1)
            if len(items) == 2 and items[0] == info:
                return items[1]
    return None


def un_download_cb(filename, command, rc, stdout, stderr):
    """Callback on download of URL."""

    if rc != 0:
        return weechat.WEECHAT_RC_OK

    weechat.bar_item_update(BAR_ITEM_NAME)

    version = get_version()
    version_num = get_version(as_number=True)
    compare_version = ""
    compare_version_num = ""
    update_avaliable = False
    global next_stable_text
    next_stable_text = ""

    # check for stable version first
    start_counting = weechat.config_get_plugin("start_counting")
    if start_counting != "":
        next_stable_date = get_info_ver("next_stable_date")
        lt = localtime()
        year, month, day = lt[0:3]  # today
        next_stable_date = next_stable_date.split("-")  # next_stable_date
        next_stable_date = (int(next_stable_date[0]), int(next_stable_date[1]),
                            int(next_stable_date[2]), 0, 0, 0, 0, 0, 0)
        next_stable_date = mktime(next_stable_date)
        today = year, month, day, 0, 0, 0, 0, 0, 0
        today = mktime(today)
        # calculate days till next_stable_date
        diff_day = (next_stable_date - today) // 60 // 60 // 24
        diff_day = "%1i" % (diff_day)

        # diff_day = 0  # test to pop up new stable text
        if (int(diff_day) > 0) and (int(diff_day) <= int(start_counting)):
            used_color = weechat.config_get_plugin("color_default")
            # TEN and counting....
            if (int(diff_day) <=
                    int(weechat.config_get_plugin("start_countdown"))):
                used_color = weechat.config_get_plugin("color_countdown")

            next_stable_text = weechat.config_get_plugin("next_stable_text")
            next_stable = get_info_ver("next_stable")
            if next_stable_text == "":
                next_stable_text = ("days left:" + weechat.color(used_color) +
                                    "%d" + weechat.color("reset") +
                                    " to stable: %s") % (
                                        int(diff_day), next_stable)
            else:
                next_stable_text = next_stable_text.replace(
                    "%d", weechat.color(used_color) + "%d" +
                    weechat.color("reset"))
                if next_stable_text.find("%s") >= 1:  # check for %s
                    next_stable_text = next_stable_text % (
                        int(diff_day), next_stable)
                else:
                    next_stable_text = next_stable_text % int(diff_day)
                update_avaliable = False
        elif (int(diff_day) <= 0):  # today a new stable version is available
            stable_number = get_info_ver("stable")
            next_stable_text = weechat.config_get_plugin("update_text_stable")
            if next_stable_text.find("%s") >= 1:  # %s in string?
                next_stable_text = (next_stable_text % stable_number)
            return weechat.WEECHAT_RC_OK

    if weechat.config_get_plugin("uses_devel") == "true":
        compare_version_num = get_info_ver("next_stable_number")
        compare_version = get_info_ver("next_stable")
        update_avaliable = int(compare_version_num) > int(version_num)
    elif weechat.config_get_plugin("uses_git") == "true":
        git_cur = get_cur_git_version()
        if git_cur is not None:  # path to git dir exists?
            git_ver = get_info_ver("git")  # yes
            compare_version = git_cur
            update_avaliable = get_cur != git_ver
            if update_avaliable:
                do_git_pull()  # call git pull
        else:
            update_avaliable = False
    else:
        compare_version_num = get_info_ver("stable_number")
        compare_version = get_info_ver("stable")
        update_avaliable = int(compare_version_num) > int(version_num)

    if update_avaliable:
        next_stable_text = weechat.config_get_plugin("update_text")

    return weechat.WEECHAT_RC_OK


def un_update_cache():
    """Callback for building update item."""
    weechat.hook_process_hashtable("url:%s" % URL_INFOS,
                                   {"file_out": get_filename_infos()},
                                   TIMEOUT_DOWNLOAD,
                                   "un_download_cb", "")


def un_day_changed(data, signal, signal_data):
    un_update_cache()
    return weechat.WEECHAT_RC_OK


def un_config_changed(data, option, value):
    un_update_cache()
    return weechat.WEECHAT_RC_OK


def up_item_cb(data, buffer, args):
    return next_stable_text


def un_cmd(data, buffer, args):
    un_update_cache()
    return weechat.WEECHAT_RC_OK


if __name__ == '__main__' and import_ok and \
       weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, default_value)

    weechat.hook_command("upgrade_check", "Checks for upgrades", "",
                         "The bar item is called \"%s\"." % BAR_ITEM_NAME,
                         "", "un_cmd", "")

    weechat.bar_item_new(BAR_ITEM_NAME, 'up_item_cb', '')
    weechat.hook_signal("day_changed", "un_day_changed", "")
    weechat.hook_config("plugins.var.python." + SCRIPT_NAME + ".*",
                        "un_config_changed", "")

    # Update the cache when it starts up.
    un_update_cache()



#
# SPDX-FileCopyrightText: 2008-2025 Sébastien Helleu <flashcode@flashtux.org>
# SPDX-FileCopyrightText: 2009 drubin <drubin@smartcube.co.za>
# SPDX-FileCopyrightText: 2010-2011 Trashlord <dornenreich666@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Tetris game for WeeChat, yeah!"""

SCRIPT_NAME = "weetris"
SCRIPT_AUTHOR = "Sébastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "1.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Tetris game for WeeChat, yeah!"

import_ok = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    import_ok = False

try:
    import random
    import time
except ImportError as exc:
    print(f"Missing package(s) for {SCRIPT_NAME}: {exc}")
    import_ok = False

BUFFER_TITLE = (
    f"{SCRIPT_NAME} {SCRIPT_VERSION} - enjoy!  |  "
    "Keys: arrows: "
    "move/rotate (ctrl-down: bottom), "
    "alt-n: new game, "
    "alt-p: pause  |  "
    "Input: q = quit"
)

GAME_WIDTH = 10
GAME_HEIGHT = 20

MAX_LEVEL = 10

START_Y = 0

PIECES = (
    1024 + 512 + 64 + 32,  # O
    2048 + 1024 + 512 + 256,  # I
    2048 + 1024 + 512 + 64,  # T
    2048 + 1024 + 512 + 128,  # L
    2048 + 1024 + 512 + 32,  # J
    1024 + 512 + 128 + 64,  # S
    2048 + 1024 + 64 + 32,  # Z
)
PIECE_COLOR = {
    # < 256 colors
    False: [
        "yellow",  # O
        "lightcyan",  # I
        "magenta",  # T
        "brown",  # L
        "blue",  # J
        "green",  # S
        "red",  # Z
    ],
    # >= 256 colors
    True: [
        "yellow",  # O
        "lightcyan",  # I
        "magenta",  # T
        "172",  # L
        "blue",  # J
        "green",  # S
        "red",  # Z
    ],
}
PIECE_X_INC = (3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0)
PIECE_Y_INC = (3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0)
PIECE_ROTATION = (
    4096,
    256,
    16,
    1,
    8192,
    512,
    32,
    2,
    16384,
    1024,
    64,
    4,
    32768,
    2048,
    128,
    8,
)

# ---------------------------------          ---------------------------------
# |       |       |       |       |          |       |       |       |       |
# | 32768 | 16384 |  8192 |  4096 |          |     8 |   128 |  2048 | 32768 |
# |       |       |       |       |          |       |       |       |       |
# ---------------------------------          ---------------------------------
# |       |       |       |       |          |       |       |       |       |
# |  2048 |  1024 |   512 |   256 |          |     4 |    64 |  1024 | 16384 |
# |       |       |       |       |  after   |       |       |       |       |
# ---------------------------------  rotate  ---------------------------------
# |       |       |       |       |          |       |       |       |       |
# |   128 |    64 |    32 |    16 |  =====>  |     2 |    32 |   512 |  8192 |
# |       |       |       |       |          |       |       |       |       |
# ---------------------------------          ---------------------------------
# |       |       |       |       |          |       |       |       |       |
# |     8 |     4 |     2 |     1 |          |     1 |    16 |   256 |  4096 |
# |       |       |       |       |          |       |       |       |       |
# ---------------------------------          ---------------------------------

weetris = {
    # will be set to True if 256 colors are supported by the terminal
    "256colors": False,
    "buffer": "",
    "timer": "",
    "level": 1,
    "matrix": [],
    "matrix_next": [],
    "playing": False,
    "paused": False,
    "lines": 0,
    "piece_x": 0,
    "piece_y": 0,
    "piece_number": -1,
    "next_piece_number": -1,
    "piece_form": 0,
    "best_level": 1,
    "best_lines": 0,
    "play_start_time": 0,
    "time_display_timer": "",
}

# script options
weetris_settings_default = {
    "display_next_piece": (
        "on",
        "display the next piece",
    ),
    "key_down_slow": (
        "on",
        "the key arrow down moves the piece slowly: one position, and ctrl+down "
        "mores directly to the bottom; if disabled, the two keys are "
        "reversed",
    ),
}
weetris_settings = {}


def weetris_config_cb(data, option, value):
    """Called when a script option is changed."""
    pos = option.rfind(".")
    if pos > 0:
        name = option[pos + 1 :]
        if name in weetris_settings:
            weetris_settings[name] = value
    return weechat.WEECHAT_RC_OK


def buffer_input_cb(data, buf, input_data):
    """Input on weetris buffer."""
    if input_data == "q":
        weechat.buffer_close(weetris["buffer"])
    return weechat.WEECHAT_RC_OK


def buffer_close_cb(data, buf):
    """Weetris buffer closed (oh no, why?)."""
    weetris["buffer"] = ""
    if weetris["timer"]:
        weechat.unhook(weetris["timer"])
        weetris["timer"] = ""
    if weetris["time_display_timer"]:
        weechat.unhook(weetris["time_display_timer"])
        weetris["play_start_time"] = 0
        weetris["time_display_timer"] = ""
    weechat.prnt("", "Thank you for playing WeeTris!")
    return weechat.WEECHAT_RC_OK


def get_piece_block(value):
    """
    Return a string with a single block of a piece
    (spaces with background color).
    """
    if value < 0:
        block = weechat.color(",default")
    else:
        block = weechat.color("," + PIECE_COLOR[weetris["256colors"]][value])
    return block + "  "


def display_line(y):
    """Display a line of the matrix."""
    line = " │"
    if weetris["paused"]:
        if y == GAME_HEIGHT // 2:
            spaces_before = ((GAME_WIDTH * 2) - 6) // 2
            spaces_after = (GAME_WIDTH * 2) - 6 - spaces_before
            line += (" " * spaces_before) + "PAUSED" + (" " * spaces_after)
        else:
            line += "  " * GAME_WIDTH
    else:
        for x in range(GAME_WIDTH):
            line += get_piece_block(weetris["matrix"][y][x])
    line += weechat.color(",default") + "│"
    if weetris["playing"] and weetris_settings["display_next_piece"] == "on":
        if y == 0:
            line += "    Next: "
        elif 1 <= y <= 4:
            line += "    "
            for x in range(4):
                line += get_piece_block(weetris["matrix_next"][y - 1][x])
            line += weechat.color(",default")
    weechat.prnt_y(weetris["buffer"], START_Y + y + 1, line)


def weetris_display_playing_time_cb(data, remaining_calls):
    """Callback of timer to display the playing time."""
    total_seconds = time.time() - weetris["play_start_time"]
    minutes = int(total_seconds // 60)
    seconds = int(total_seconds % 60)
    total_seconds += 1
    weechat.prnt_y(
        weetris["buffer"],
        START_Y + GAME_HEIGHT + 6,
        f" Playing time : {minutes:02d}:{seconds:02d}",
    )
    return weechat.WEECHAT_RC_OK


def display_level_lines():
    """Display the current level and number of lines."""
    str_lines = "lines" if weetris["lines"] > 1 else "line"
    list_info = [
        f"Level {weetris['level']:-3d} {weetris['lines']:6d} {str_lines}",
        "-" * (1 + (GAME_WIDTH * 2) + 1),
        f"Highest level: {weetris['best_level']}",
        f"Max lines    : {weetris['best_lines']}",
    ]
    for y, info in enumerate(list_info):
        weechat.prnt_y(weetris["buffer"], START_Y + GAME_HEIGHT + 2 + y, " " + info)


def display_piece(display):
    """Display (or hide) the current piece."""
    value = weetris["piece_number"] if display else -1
    for i in range(16):
        if weetris["piece_form"] & (1 << i):
            x2 = weetris["piece_x"] + PIECE_X_INC[i]
            y2 = weetris["piece_y"] + PIECE_Y_INC[i]
            weetris["matrix"][y2][x2] = value


def display_all():
    """Display everything on the weetris buffer."""
    display_piece(True)
    weechat.prnt_y(weetris["buffer"], START_Y, " ┌" + ("──" * GAME_WIDTH) + "┐")
    for y in range(GAME_HEIGHT):
        display_line(y)
    weechat.prnt_y(
        weetris["buffer"], START_Y + GAME_HEIGHT + 1, " └" + ("──" * GAME_WIDTH) + "┘"
    )
    display_piece(False)


def random_piece():
    """Return a random piece number."""
    return random.randint(0, len(PIECES) - 1)


def set_matrix_next():
    """Set the matrix for the next piece."""
    weetris["matrix_next"] = [[-1] * 4 for i in range(4)]
    number = weetris["next_piece_number"]
    form = PIECES[number]
    for i in range(16):
        if form & (1 << i):
            weetris["matrix_next"][PIECE_Y_INC[i]][PIECE_X_INC[i]] = number


def set_new_form():
    """Choose a new random form."""
    if weetris["next_piece_number"] < 0:
        weetris["next_piece_number"] = random_piece()
    weetris["piece_number"] = weetris["next_piece_number"]
    weetris["next_piece_number"] = random_piece()
    set_matrix_next()
    weetris["piece_form"] = PIECES[weetris["piece_number"]]
    weetris["piece_x"] = (GAME_WIDTH // 2) - 2
    weetris["piece_y"] = 0


def init_timer():
    """Initialize timer."""
    if weetris["timer"]:
        weechat.unhook(weetris["timer"])
    delay = max(100, 700 - ((weetris["level"] - 1) * 60))
    weetris["timer"] = weechat.hook_timer(delay, 0, 0, "weetris_timer_cb", "")


def new_game():
    """New game."""
    weechat.prnt_y(weetris["buffer"], START_Y + GAME_HEIGHT + 2, "")
    weetris["matrix"] = [[-1] * GAME_WIDTH for i in range(GAME_HEIGHT)]
    weetris["next_piece_number"] = -1
    set_new_form()
    weetris["playing"] = True
    weetris["paused"] = False
    weetris["lines"] = 0
    weetris["level"] = 1
    weetris["play_start_time"] = time.time()
    weechat.prnt_y(
        weetris["buffer"], START_Y + GAME_HEIGHT + 6, " Playing time : 00:00"
    )
    init_timer()
    weetris["time_display_timer"] = weechat.hook_timer(
        1000,
        0,
        0,
        "weetris_display_playing_time_cb",
        "",
    )
    display_all()
    display_level_lines()


def rotation(form):
    """Rotate a form."""
    new_form = 0
    for i in range(16):
        if form & (1 << i):
            new_form |= PIECE_ROTATION[i]
    return new_form


def is_possible(new_x, new_y, new_form):
    """Check if the "new_form" can be moved to position (new_x, new_y)."""
    for i in range(16):
        if not new_form & (1 << i):
            continue
        x = new_x + PIECE_X_INC[i]
        y = new_y + PIECE_Y_INC[i]
        if (
            x < 0
            or x >= GAME_WIDTH
            or y < 0
            or y >= GAME_HEIGHT
            or weetris["matrix"][y][x] >= 0
        ):
            return 0
    return 1


def remove_completed_lines():
    """Remove completed lines."""
    y = GAME_HEIGHT - 1
    lines_removed = False
    while y >= 0:
        if -1 not in weetris["matrix"][y]:
            for i in range(y, -1, -1):
                if i == 0:
                    weetris["matrix"][i] = [-1] * GAME_WIDTH
                else:
                    weetris["matrix"][i] = weetris["matrix"][i - 1]
            # Removes the line and increases the number of lines made
            # in the game in "lines"
            weetris["lines"] += 1
            lines_removed = True
            if weetris["lines"] > weetris["best_lines"]:
                set_best("max_lines", weetris["lines"])
                weetris["best_lines"] = weetris["lines"]
        else:
            y -= 1
    if lines_removed:
        new_level = min(MAX_LEVEL, (weetris["lines"] // 10) + 1)
        if new_level != weetris["level"]:
            # Next level
            weetris["level"] = new_level
            if weetris["level"] > weetris["best_level"]:
                set_best("max_level", weetris["level"])
                weetris["best_level"] = weetris["level"]
            init_timer()
        display_level_lines()


def end_of_piece():
    """End of a piece (it can not go down any more)."""
    display_piece(True)
    set_new_form()
    if is_possible(weetris["piece_x"], weetris["piece_y"], weetris["piece_form"]):
        remove_completed_lines()
    else:
        weetris["piece_form"] = 0
        weetris["playing"] = False
        weetris["paused"] = False
        if weetris["time_display_timer"]:
            weechat.unhook(weetris["time_display_timer"])
            weetris["time_display_timer"] = ""
        weechat.prnt_y(
            weetris["buffer"],
            START_Y + GAME_HEIGHT + 2,
            f">> End of game, score: {weetris['lines']} lines, "
            f"level weetris['level'] (alt-n to restart) <<"
        )


def weetris_init():
    """Initialize weetris."""
    keys = {
        "up": "up",
        "down": "down",
        "ctrl-down": "bottom",
        "left": "left",
        "right": "right",
        "meta-n": "new_game",
        "meta-p": "pause",
    }
    weetris["buffer"] = weechat.buffer_search("python", "weetris")
    if not weetris["buffer"]:
        weetris["buffer"] = weechat.buffer_new(
            "weetris", "buffer_input_cb", "", "buffer_close_cb", ""
        )
    if weetris["buffer"]:
        weechat.buffer_set(weetris["buffer"], "type", "free")
        weechat.buffer_set(weetris["buffer"], "title", BUFFER_TITLE)
        for key, action in keys.items():
            weechat.buffer_set(
                weetris["buffer"], f"key_bind_{key}", f"/weetris {action}"
            )
        new_game()
        weechat.buffer_set(weetris["buffer"], "display", "1")


def run_action(action):
    """Run an action, when a key is pressed on weetris buffer."""
    if action == "rotate":
        new_form = rotation(weetris["piece_form"])
        if is_possible(weetris["piece_x"], weetris["piece_y"], new_form):
            weetris["piece_form"] = new_form
            display_all()
    elif action == "left":
        if is_possible(
            weetris["piece_x"] - 1, weetris["piece_y"], weetris["piece_form"]
        ):
            weetris["piece_x"] -= 1
            display_all()
    elif action == "right":
        if is_possible(
            weetris["piece_x"] + 1, weetris["piece_y"], weetris["piece_form"]
        ):
            weetris["piece_x"] += 1
            display_all()
    elif action == "down":
        if is_possible(
            weetris["piece_x"], weetris["piece_y"] + 1, weetris["piece_form"]
        ):
            weetris["piece_y"] += 1
        else:
            end_of_piece()
        display_all()
    elif action == "bottom":
        while is_possible(
            weetris["piece_x"], weetris["piece_y"] + 1, weetris["piece_form"]
        ):
            weetris["piece_y"] += 1
        end_of_piece()
        display_all()


def weetris_cmd_cb(data, buf, args):
    """Callback for command /weetris."""
    if weetris["buffer"]:
        weechat.buffer_set(weetris["buffer"], "display", "1")
    else:
        weetris_init()

    if args == "new_game":
        new_game()
    elif args == "pause":
        if weetris["playing"]:
            weetris["paused"] = not weetris["paused"]
            display_all()
    else:
        if weetris["playing"] and not weetris["paused"]:
            key_slow_down = weetris_settings["key_down_slow"] == "on"
            actions = {
                "up": "rotate",
                "left": "left",
                "right": "right",
                "down": "down" if key_slow_down else "bottom",
                "bottom": "bottom" if key_slow_down else "down",
            }
            action = actions.get(args)
            if action:
                run_action(action)
    return weechat.WEECHAT_RC_OK


def weetris_timer_cb(data, remaining_calls):
    """Weetris timer callback."""
    if weetris["buffer"] and weetris["playing"] and not weetris["paused"]:
        if is_possible(
            weetris["piece_x"], weetris["piece_y"] + 1, weetris["piece_form"]
        ):
            weetris["piece_y"] += 1
        else:
            end_of_piece()
        display_all()
    return weechat.WEECHAT_RC_OK


def get_best(name, default=0):
    """Get the best level/lines."""
    value = weechat.config_get_plugin(name)
    return int(value) if value else default


def set_best(name, value):
    """Set the best level/lines."""
    weechat.config_set_plugin(name, str(value))


def main():
    """Main function."""
    if weechat.register(
        SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""
    ):
        term_colors = int(weechat.info_get("term_colors", "") or "8")
        weetris["256colors"] = term_colors >= 256

        # set default settings
        version = weechat.info_get("version_number", "") or 0
        for option, value in weetris_settings_default.items():
            if weechat.config_is_set_plugin(option):
                weetris_settings[option] = weechat.config_get_plugin(option)
            else:
                weechat.config_set_plugin(option, value[0])
                weetris_settings[option] = value[0]
            if int(version) >= 0x00030500:
                weechat.config_set_desc_plugin(
                    option, f"{value[1]} (default: \"{value[0]}\")"
                )

        # detect config changes
        weechat.hook_config(
            f"plugins.var.python.{SCRIPT_NAME}.*", "weetris_config_cb", ""
        )

        # command /weetris
        weechat.hook_command(
            "weetris",
            "Run WeeTris",
            "",
            "Keys:\n"
            "   arrow up: rotate current piece\n"
            " arrow left: move piece to the left\n"
            "arrow right: move piece to the right\n"
            " arrow down: increase speed of the piece\n"
            "  ctrl+down: move piece to the bottom\n"
            "      alt+n: restart the game\n"
            "      alt+p: pause current game",
            "",
            "weetris_cmd_cb",
            "",
        )

        # initialization
        if weechat.buffer_search("python", "weetris"):
            weetris_init()
        weetris["best_level"] = get_best("max_level", 1)
        weetris["best_lines"] = get_best("max_lines", 0)


if __name__ == "__main__" and import_ok:
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
# Borrowed parts from pagetitle.py by xororand
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
#
# If someone posts an URL in a configured channel
# this script will post back title

# Explanation about ignores:
#   * plugins.var.python.announce_url_title.ignore_buffers:
#   Comma separated list of patterns for define ignores.
#   URLs from channels where its name matches any of these patterns will be
#   ignored.
#   Wildcards '*', '?' and char groups [..] can be used.
#   An ignore exception can be added by prefixing '!' in the pattern.
#
#       Example:
#       *ubuntu*,!#ubuntu-offtopic
#       any urls from a 'ubuntu' channel will be ignored,
#       except from #ubuntu-offtopic
#
#   * plugins.var.python.announce_url_title.url_ignore
#     simply does partial match, so specifying 'google' will ignore every url
#     with the word google in it
#
#
# History:
#
# 2021-06-05, Sébastien Helleu <flashcode@flashtux.org>
#   version 19: make script compatible with Python 3, fix PEP8 errors
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#   version 18: change hook_print callback argument type of displayed/highlight
#               (WeeChat >= 1.0)
# 2013-11-07, excalibr
#   version 17: add more characters to exclude in escaping (this fix problem
#               with youtube urls)
# 2012-11-15, xt
#   version 16: improve escaping
# 2011-09-04, Deltafire
#   version 15: fix remote execution exploit due to unescaped ' character in
#               urls; small bug fix for version 14 changes
# 2011-08-23, Deltafire
#   version 14: ignore filtered lines
# 2011-03-11, Sébastien Helleu <flashcode@flashtux.org>
#   version 13: get python 2.x binary for hook_process (fix problem when
#               python 3.x is default python version)
# 2010-12-10, xt
#   version 12: add better ignores (code based on m4v inotify.py)
# 2010-11-02, xt
#   version 11: add prefix
# 2010-11-01, xt
#   version 10: add ignored buffers feature
# 2010-10-29, add ignore buffers feature
#   version 0.9: WeeChat user-agent option
# 2010-10-11, xt
#   version 0.8: support multiple concurrent url lookups
# 2010-10-11, xt
#   version 0.7: do not trigger on notices
# 2010-08-25, xt
#   version 0.6: notice some buffers instead of msg
# 2009-12-08, Chaz6
#   version 0.5: only announce for specified channels
# 2009-12-08, Chaz6 <chaz@chaz6.com>
#   version 0.4: add global option
# 2009-12-08, xt
#   version 0.3: option for public announcing or not
# 2009-12-07, xt <xt@bash.no>
#   version 0.2: don't renannounce same urls for a time
#                add optional prefix and suffix
# 2009-12-02, xt
#   version 0.1: initial

from time import time as now
from fnmatch import fnmatch
from html import unescape
from urllib.error import URLError
from urllib.parse import quote
from urllib.request import Request, urlopen

import weechat
import re

SCRIPT_NAME = "announce_url_title"
SCRIPT_AUTHOR = "xt <xt@bash.no>"
SCRIPT_VERSION = "19"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Announce URL titles to channel or locally"

settings = {
    # comma separated list of buffers
    "buffers": "libera.#testing,",
    # comma separated list of buffers
    "buffers_notice": "libera.#testing,",
    # comma separated list of buffers to be ignored by this module
    "ignore_buffers": "grep,",
    "title_max_length": "80",
    # comma separated list of strings in url to ignore
    "url_ignore": "",
    # 5 minutes delay
    "reannounce_wait": "5",
    "prefix": "",
    "suffix": "",
    # print it or msg the buffer
    "announce_public": "off",
    # whether to enable for all buffers
    "global": "off",
    # user-agent format string
    "user_agent": "WeeChat/%(version)s (https://weechat.org)",
    # Prefix for when not public announcement
    "global_prefix": "url",
}


octet = r"(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})"
ipAddr = r"%s(?:\,.%s){3}" % (octet, octet)
# Base domain regex off RFC 1034 and 1738
label = r"[0-9a-z][-0-9a-z]*[0-9a-z]?"
domain = r"%s(?:\.%s)*\.[a-z][-0-9a-z]*[a-z]?" % (label, label)
urlRe = re.compile(
    r"(\w+://(?:%s|%s)(?::\d+)?(?:/[^\])>\s]*)?)" % (domain, ipAddr), re.I
)

buffer_name = ""

urls = {}
script_nick = "url"


def say(s, buffer=""):
    """Display message."""
    weechat.prnt(buffer, "%s\t%s" % (script_nick, s))


def url_print_cb(
    data, buffer, time, tags, displayed, highlight, prefix, message
):
    global buffer_name, urls, ignore_buffers

    # Do not trigger on filtered lines and notices
    if not int(displayed) or prefix == "--":
        return weechat.WEECHAT_RC_OK

    msg_buffer_name = weechat.buffer_get_string(buffer, "name")

    # Skip ignored buffers
    if msg_buffer_name in ignore_buffers:
        return weechat.WEECHAT_RC_OK

    found = False
    if weechat.config_get_plugin("global") == "on":
        found = True
        buffer_name = msg_buffer_name
    else:
        buffers = weechat.config_get_plugin("buffers").split(",")
        for active_buffer in buffers:
            if active_buffer.lower() == msg_buffer_name.lower():
                found = True
                buffer_name = msg_buffer_name
                break
        buffers_notice = weechat.config_get_plugin("buffers_notice").split(",")
        for active_buffer in buffers_notice:
            if active_buffer.lower() == msg_buffer_name.lower():
                found = True
                buffer_name = msg_buffer_name
                break

    if not found:
        return weechat.WEECHAT_RC_OK

    ignorelist = weechat.config_get_plugin("url_ignore").split(",")
    for url in urlRe.findall(message):

        url_esc = quote(url, "%/:=&?~#+!$,;@()*[]")  # Escape URL
        ignore = False
        for ignore_part in ignorelist:
            if ignore_part.strip():
                if ignore_part in url_esc:
                    ignore = True
                    weechat.prnt(
                        "",
                        "%s: Found %s in URL: %s, ignoring."
                        % (SCRIPT_NAME, ignore_part, url_esc),
                    )
                    break

        if ignore:
            continue

        if url_esc in urls:
            continue
        else:
            urls[url_esc] = {}

    url_process_launcher()

    return weechat.WEECHAT_RC_OK


def url_read(url):
    """Read URL."""
    user_agent = weechat.config_get_plugin("user_agent") % {
        "version": weechat.info_get("version", "")
    }
    req = Request(
        url,
        headers={
            "User-agent": user_agent,
        },
    )
    try:
        head = urlopen(req).read(8192).decode("utf-8", errors="ignore")
    except URLError:
        return ""
    match = re.search("(?i)<title>(.*?)</title>", head)
    return unescape(match.group(1)) if match else ""


def url_process_cb(data, command, rc, stdout, stderr):
    """Process callback."""

    global buffer_name, urls

    title = stdout
    max_len = int(weechat.config_get_plugin("title_max_length"))
    if len(title) > max_len:
        title = "%s [...]" % title[0:max_len]

    splits = buffer_name.split(".")  # FIXME bad code
    server = splits[0]
    buffer = ".".join(splits[1:])
    output = (
        weechat.config_get_plugin("prefix")
        + title
        + weechat.config_get_plugin("suffix")
    )
    announce_public = weechat.config_get_plugin("announce_public")
    if announce_public == "on":
        found = False
        buffers = weechat.config_get_plugin("buffers").split(",")
        for active_buffer in buffers:
            if active_buffer.lower() == buffer_name.lower():
                weechat.command(
                    "",
                    "/msg -server %s %s %s" % (server, buffer, output),
                )
                found = True
        buffers_notice = weechat.config_get_plugin("buffers_notice").split(",")
        for active_buffer in buffers_notice:
            if active_buffer.lower() == buffer_name.lower():
                weechat.command(
                    "",
                    "/notice -server %s %s %s" % (server, buffer, output),
                )
                found = True
        if not found:
            say(output, weechat.buffer_search("", buffer_name))
    else:
        say(output, weechat.buffer_search("", buffer_name))

    return weechat.WEECHAT_RC_OK


def url_process_launcher():
    """Iterate found urls, fetch title if hasn't been launched."""
    global urls

    for url, url_d in urls.items():
        if not url_d:  # empty dict means not launched
            url_d["launched"] = now()
            url_d["url_hook_process"] = weechat.hook_process(
                "func:url_read",
                30 * 1000,
                "url_process_cb",
                url,
            )

    return weechat.WEECHAT_RC_OK


def purge_cb(*args):
    """Purge the url list on configured intervals."""

    global urls

    t_now = now()
    reannounce_wait = int(weechat.config_get_plugin("reannounce_wait")) * 60
    for url in list(urls):
        if t_now - urls[url]["launched"] > reannounce_wait:
            del urls[url]

    return weechat.WEECHAT_RC_OK


class Ignores(object):
    def __init__(self, ignore_type):
        self.ignore_type = ignore_type
        self.ignores = []
        self.exceptions = []
        self._get_ignores()

    def _get_ignores(self):
        assert self.ignore_type is not None
        ignores = weechat.config_get_plugin(self.ignore_type).split(",")
        ignores = [s.lower() for s in ignores if s]
        self.ignores = [s for s in ignores if s[0] != "!"]
        self.exceptions = [s[1:] for s in ignores if s[0] == "!"]

    def __contains__(self, s):
        s = s.lower()
        for p in self.ignores:
            if fnmatch(s, p):
                for e in self.exceptions:
                    if fnmatch(s, e):
                        return False
                return True
        return False


def ignore_update(*args):
    ignore_buffers._get_ignores()
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__":
    if weechat.register(
        SCRIPT_NAME,
        SCRIPT_AUTHOR,
        SCRIPT_VERSION,
        SCRIPT_LICENSE,
        SCRIPT_DESC,
        "",
        "",
    ):

        # Set default settings
        for option, default_value in settings.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)
        ignore_buffers = Ignores("ignore_buffers")

        weechat.hook_print("", "", "://", 1, "url_print_cb", "")
        weechat.hook_timer(
            int(weechat.config_get_plugin("reannounce_wait")) * 1000 * 60,
            0,
            0,
            "purge_cb",
            "",
        )
        weechat.hook_config(
            "plugins.var.python.%s.ignore_buffers" % SCRIPT_NAME,
            "ignore_update",
            "",
        )
    color_chat_delimiters = weechat.color("chat_delimiters")
    color_chat_nick = weechat.color("chat_nick")
    color_reset = weechat.color("reset")
    color_chat_buffer = weechat.color("chat_buffer")
    # pretty printing
    script_nick = "%s[%s%s%s]%s" % (
        color_chat_delimiters,
        color_chat_nick,
        weechat.config_get_plugin("global_prefix"),
        color_chat_delimiters,
        color_reset,
    )



# -*- coding: utf-8 -*-
# Add tab completion to bitlbee commands
# based on http://scripts.irssi.org/scripts/bitlbee_tab_completion.pl
#
# History:
#
# 2023-10-04, Andrea Beciani <andrea.beciani.0@gmail.com>:
#     version 0.3: set default template, fix command executed, fix function names
# 2015-11-02, Mickaël Thomas <mickael9@gmail.com>:
#     version 0.2: strip color attributes for topic detection
# 2015-03-22, Roger Duran <rogerduran@gmail.com>:
#     version 0.1: initial version

import weechat

SCRIPT_NAME = "bitlbee_completion"
SCRIPT_AUTHOR = "Roger Duran <rogerduran@gmail.com>"
SCRIPT_VERSION = "0.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Add tab completion to bitlbee commands"
TEMPLATE_NAME = "bitlbee_completion"

OPTS = {
    "server": None,
    "channel": None
    }

TOPIC = "Welcome to the control channel. "\
    "Type help for help information."

commands = []


def request_completion():
    """
    Request the completion to the bitlbee server and wait for response
    """
    server = OPTS["server"]
    weechat.command("", "/quote -server %s COMPLETIONS" % server)


def modifier_cb(data, modifier, modifier_data, string):
    """
    When the server returns the completion, update the commands list
    """

    if ":COMPLETIONS" not in string:
        return string
    command = string.split(":COMPLETIONS ")[1]
    if command not in ("OK", "END"):
        commands.append(command)
    return ""


def completion_cb(data, completion_item, buffer, completion):
    """
    Complete bitlbee commands only in the bitlbee buffer
    """

    server = OPTS["server"]
    channel = OPTS["channel"]
    if not server or not channel:
        return weechat.WEECHAT_RC_OK

    buff_name = weechat.buffer_get_string(buffer, "name")
    if buff_name == "%s.%s" % (server, channel):
        for command in commands:
            weechat.hook_completion_list_add(completion, command, 0,
                                             weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def check_config():
    option = weechat.config_get("weechat.completion.default_template")
    default_template = weechat.config_string(option)
    if TEMPLATE_NAME not in default_template:
        rc = weechat.config_option_set(option, default_template + "|%(" + TEMPLATE_NAME + ")", 1)
        if rc == weechat.WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE:
            weechat.prnt("", SCRIPT_NAME + " -  warning! - weechat.completion.default_template same value")
        elif rc == weechat.WEECHAT_CONFIG_OPTION_SET_ERROR:
            weechat.prnt("", SCRIPT_NAME + " -  error! - writing weechat.completion.default_template option")

def find_buffer():
    """
    Find the buffer when the plugin starts
    """
    infolist = weechat.infolist_get("buffer", "", "")
    while weechat.infolist_next(infolist):
        topic = weechat.infolist_string(infolist, "title")
        if weechat.string_remove_color(topic, "") == TOPIC:
            name = weechat.infolist_string(infolist, "name")
            set_options(name)
            request_completion()
            break
    weechat.infolist_free(infolist)


def set_options(name):
    server, channel = name.split(".")
    OPTS["server"] = server
    OPTS["channel"] = channel


def print_cb(data, buffer, time, tags, displayed, highlight, prefix, message):
    """
    Find the buffer when a new one is open
    """
    current_topic = weechat.string_remove_color(message, "").split('"')[1]
    if current_topic == TOPIC:
        name = weechat.buffer_get_string(buffer, "name")
        set_options(name)
        request_completion()
    return weechat.WEECHAT_RC_OK


def main():
    check_config()
    weechat.hook_modifier("irc_in_notice", "modifier_cb", "")
    weechat.hook_completion(TEMPLATE_NAME, "TAB completion to bitlbee",
                            "completion_cb", "")

    weechat.hook_print('', 'irc_332', '', 1, 'print_cb', '')
    weechat.hook_print('', 'irc_topic', '', 1, 'print_cb', '')
    find_buffer()

if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
# USAGE: Bind a key to command /flip . Then write some text at input line
# press your key to transform it to upside down.

#
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2022-03-14, TauPan <taupan@gmail.com>
#   version 0.3.1: port to python3
# 2010-01-14, xt
#   version 0.3: steal more chars from m4v
# 2010-01-06, xt <xt@bash.no>
#   version 0.2: fix idiotic programming
# 2009-11-12, xt <xt@bash.no>
#   version 0.1: initial release

import weechat as w
import re

SCRIPT_NAME    = "upside_down"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.3.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Replaces text you write with upside down text"

settings = {}

replacements = {
# Upper case
    u'A' : u'\N{FOR ALL}',
    u'B' : u'\N{GREEK SMALL LETTER XI}',
    u'C' : u'\N{ROMAN NUMERAL REVERSED ONE HUNDRED}',
    u'D' : u'\N{LEFT HALF BLACK CIRCLE}',
    u'E' : u'\N{LATIN CAPITAL LETTER REVERSED E}',
    u'F' : u'\N{TURNED CAPITAL F}',
    u'G' : u'\N{TURNED SANS-SERIF CAPITAL G}',
    u'J' : u'\N{LATIN SMALL LETTER LONG S}',
    u'K' : u'\N{RIGHT NORMAL FACTOR SEMIDIRECT PRODUCT}',
    u'L' : u'\ua780',
    u'M' : u'W',
    u'N' : u'\N{LATIN LETTER SMALL CAPITAL REVERSED N}',
    u'P' : u'\N{CYRILLIC CAPITAL LETTER KOMI DE}',
    u'Q' : u'\N{GREEK CAPITAL LETTER OMICRON WITH TONOS}',
    u'R' : u'\N{LATIN LETTER SMALL CAPITAL TURNED R}',
    u'T' : u'\N{UP TACK}',
    u'U' : u'\N{INTERSECTION}',
    u'V' : u'\u0245',
    u'Y' : u'\N{TURNED SANS-SERIF CAPITAL Y}',
     'a' : u"\u0250",
     'b' : u'q',
     'c' : u"\u0254",
     'd' : u'p',
     'e' : u"\u01DD",
     'f' : u"\u025F",
     'g' : u"\u0183",
     'h' : u'\u0265',
     'i' : u'\u0131',
     'j' : u'\u027E',
     'k' : u'\u029E',
     'm' : u'\u026F',
     'n' : u'u',
     'r' : u'\u0279',
     't' : u'\u0287',
     'p' : u'd',
     'u' : u'n',
     'q' : u'b',
     'v' : u'\u028C',
     'w' : u'\u028D',
     'y' : u'\u028E',
     '.' : u'\u02D9',
     '[' : u']',
     '(' : u')',
     '{' : u'}',
     '?' : u'\u00BF',
     '!' : u'\u00A1',
     "\'" :u',',
     '>' : u'<',
     '<' : u'>',
     '_' : u'\u203E',
     ';' : u'\u061B',
     '\u203F' : u'\u2040',
     '\u2045' : u'\u2046',
     '\u2234' : u'\u2235',
}



if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)
    w.hook_command("flip",
                         SCRIPT_DESC,
                         "[text]",
                         "text: text to be flipped\n"
                         "",
                         "", "flip_cmd_cb", "")


def flip_cmd_cb(data, buffer, args):
    ''' Command /flip '''
    translate_input = args
    if not translate_input:
        translate_input = w.buffer_get_string(w.current_buffer(), "input")
    outstring = ''
    for char  in translate_input:
        if char in replacements:
            char = replacements[char]
        outstring += char
    w.buffer_set(w.current_buffer(), 'input', outstring)
    return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 by Simmo Saan <simmo.saan@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# History:
#
# 2021-11-06, Sébastien Helleu <flashcode@flashtux.org>
#   version 0.3: make script compatible with WeeChat >= 3.4
#                (new parameters in function hdata_search)
# 2019-10-20, Simmo Saan <simmo.saan@gmail.com>
#   version 0.2: improve script description
# 2019-10-18, Simmo Saan <simmo.saan@gmail.com>
#   version 0.1: initial script
#

"""
Open buffers by full name, reopen closed recently closed buffers, open layout buffers
"""

# Adding handler for other full names in this script (only if necessary) or other scripts (preferred):
#
# def buffer_open_full_name_cb(data, signal, hashtable):
#     full_name = hashtable["full_name"]
#     noswitch = bool(int(hashtable.get("noswitch", "0")))
#
#     if full_name == "my.buffer.full.name":
#         # open my.buffer.full.name, considering noswitch if possible
#         return weechat.WEECHAT_RC_OK_EAT  # prevent other callbacks from handling this full name
#
#     return weechat.WEECHAT_RC_OK  # let other callbacks handle this full name
#
# weechat.hook_hsignal("buffer_open_full_name", "buffer_open_full_name_cb", "")

from __future__ import print_function

SCRIPT_NAME = "buffer_open"
SCRIPT_AUTHOR = "Simmo Saan <simmo.saan@gmail.com>"
SCRIPT_VERSION = "0.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Open buffers by full name, reopen closed recently closed buffers, open layout buffers"

SCRIPT_REPO = "https://github.com/sim642/buffer_open"

SCRIPT_COMMAND = SCRIPT_NAME

IMPORT_OK = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    IMPORT_OK = False

import re
import collections


SETTINGS = {
    "layout_apply": (
        "off",
        "open closed layout buffers on /layout apply"
    ),
    "max_closed": (
        "10",
        "maximum number of closed buffers to remember"
    )
}


def log(string):
    weechat.prnt("", "{}: {}".format(SCRIPT_NAME, string))


def error(string):
    weechat.prnt("", "{}{}: {}".format(weechat.prefix("error"), SCRIPT_NAME, string))


def command_plugin(plugin, command):
    weechat.command("", "/command {} {}".format(plugin, command))


def buffer_open_full_name_opened_cb(data, signal, hashtable):
    full_name = hashtable["full_name"]
    buffer = weechat.buffer_search("==", full_name)
    if buffer:
        # already open, do nothing
        return weechat.WEECHAT_RC_OK_EAT

    return weechat.WEECHAT_RC_OK


def buffer_open_full_name_unhandled_cb(data, signal, hashtable):
    full_name = hashtable["full_name"]
    error("no handler for opening buffer {}".format(full_name))
    return weechat.WEECHAT_RC_OK


TABLE = {
    # "full_name": ("plugin", "command"),
    "core.secured_data": ("core", "/secure"),
    "core.color": ("core", "/color"),
    "core.weechat": ("core", ""),  # do nothing because always open
    "fset.fset": ("fset", "/fset"),
    "irc.irc_raw": ("irc", "/server raw"),
    "relay.relay.list": ("relay", "/relay"),
    "relay.relay_raw": ("relay", "/relay raw"),
    "script.scripts": ("script", "/script"),
    "trigger.monitor": ("trigger", "/trigger monitor"),
    "xfer.xfer.list": ("xfer", "/xfer"),  # TODO: xfer DCC chat buffer
}


def buffer_open_full_name_table_cb(data, signal, hashtable):
    full_name = hashtable["full_name"]

    if full_name in TABLE:
        plugin, command = TABLE[full_name]
        command_plugin(plugin, command)
        return weechat.WEECHAT_RC_OK_EAT

    return weechat.WEECHAT_RC_OK


IRC_SERVER_RE = re.compile(r"^irc\.server\.(.+)$")
IRC_BUFFER_RE = re.compile(r"^irc\.([^.]+)\.(.+)$")

irc_server_connected_opens = collections.defaultdict(set)


def irc_server_open(server, noswitch):
    command_plugin("irc", "/connect {}".format(server))  # /connect doesn't have -noswitch


def hdata_search(hdata, pointer, expr, name):
    weechat_version = int(weechat.info_get("version_number", "") or 0)
    if weechat_version >= 0x03040000:
        return weechat.hdata_search(
            hdata,
            pointer,
            expr + " == ${name}",
            {},
            {"name": name},
            {},
            1,
        )
    return weechat.hdata_search(hdata, pointer, expr + " == " + name, 1)


def irc_buffer_open(server, name, noswitch):
    hdata_irc_server = weechat.hdata_get("irc_server")
    irc_servers = weechat.hdata_get_list(hdata_irc_server, "irc_servers")
    irc_server = hdata_search(hdata_irc_server, irc_servers,
                              "${irc_server.name}", server)
    chantypes = weechat.hdata_string(hdata_irc_server, irc_server, "chantypes")
    is_channel = name[0] in chantypes

    noswitch_flag = "-noswitch " if noswitch else ""
    if is_channel:
        command_plugin("irc", "/join {}-server {} {}".format(noswitch_flag, server, name))
    else:
        command_plugin("irc", "/query {}-server {} {}".format(noswitch_flag, server, name))


def irc_server_connected_cb(data, signal, server):
    for name, noswitch in irc_server_connected_opens[server]:
        irc_buffer_open(server, name, noswitch)

    irc_server_connected_opens[server] = set()
    return weechat.WEECHAT_RC_OK


def buffer_open_full_name_irc_cb(data, signal, hashtable):
    full_name = hashtable["full_name"]
    noswitch = bool(int(hashtable.get("noswitch", "0")))

    m = IRC_SERVER_RE.match(full_name)
    if m:
        server = m.group(1)
        irc_server_open(server, noswitch)
        return weechat.WEECHAT_RC_OK_EAT

    m = IRC_BUFFER_RE.match(full_name)
    if m:
        server = m.group(1)
        name = m.group(2)

        hdata_irc_server = weechat.hdata_get("irc_server")
        irc_servers = weechat.hdata_get_list(hdata_irc_server, "irc_servers")
        irc_server = hdata_search(hdata_irc_server, irc_servers,
                                  "${irc_server.name} == ", server)
        if irc_server:
            is_connected = bool(weechat.hdata_integer(hdata_irc_server, irc_server, "is_connected"))
            is_connecting = bool(weechat.hdata_pointer(hdata_irc_server, irc_server, "hook_connect"))
            if is_connected:
                irc_buffer_open(server, name, noswitch)
            else:
                irc_server_connected_opens[server].add((name, noswitch))
                if not is_connecting:
                    irc_server_open(server, noswitch)
        else:
            error("unknown server {}".format(server))

        return weechat.WEECHAT_RC_OK_EAT

    return weechat.WEECHAT_RC_OK


def buffer_open_full_name(full_name, noswitch=None):
    hashtable = {
        "full_name": full_name
    }
    if noswitch is not None:
        hashtable["noswitch"] = str(int(noswitch))  # must be str for API

    weechat.hook_hsignal_send("buffer_open_full_name", hashtable)


def command_cb(data, buffer, args):
    args = args.split()
    if len(args) >= 1 and args[0] == "closed":
        if len(args) >= 2 and args[1] == "-list":
            if buffer_closed_stack:
                weechat.prnt("", "closed buffers (latest first):")
                for full_name in reversed(buffer_closed_stack):
                    weechat.prnt("", "  {}".format(full_name))
            else:
                weechat.prnt("", "no known closed buffers")
        else:
            if buffer_closed_stack:
                noswitch = len(args) >= 2 and args[1] == "-noswitch"
                full_name = buffer_closed_stack.pop()
                buffer_open_full_name(full_name, noswitch=noswitch)
            else:
                error("no known closed buffers")
                return weechat.WEECHAT_RC_ERROR
    elif len(args) >= 1:
        noswitch = args[0] == "-noswitch"
        if noswitch:
            if len(args) >= 2:
                full_name = args[1]
            else:
                error("missing full name")
                return weechat.WEECHAT_RC_ERROR
        else:
            full_name = args[0]
        buffer_open_full_name(full_name, noswitch=noswitch)
    else:
        error("unknown subcommand")
        return weechat.WEECHAT_RC_ERROR

    return weechat.WEECHAT_RC_OK


buffer_closed_stack = []


def buffer_closing_cb(data, signal, buffer):
    global buffer_closed_stack

    full_name = weechat.buffer_get_string(buffer, "full_name")
    buffer_closed_stack.append(full_name)

    max_closed = int(weechat.config_get_plugin("max_closed"))
    buffer_closed_stack = buffer_closed_stack[max(0, len(buffer_closed_stack) - max_closed):]
    return weechat.WEECHAT_RC_OK


LAYOUT_APPLY_RE = re.compile(r"^/layout apply(?:\s+(\S+)(?:\s+buffers)?)?$")


def layout_apply_cb(data, buffer, command):
    if weechat.config_string_to_boolean(weechat.config_get_plugin("layout_apply")):
        m = LAYOUT_APPLY_RE.match(command)
        if m:
            layout_name = m.group(1) or "default"
            hdata_layout = weechat.hdata_get("layout")
            layouts = weechat.hdata_get_list(hdata_layout, "gui_layouts")
            layout = hdata_search(hdata_layout, layouts,
                                  "${layout.name}", layout_name)
            if layout:
                hdata_layout_buffer = weechat.hdata_get("layout_buffer")
                layout_buffer = weechat.hdata_pointer(hdata_layout, layout, "layout_buffers")
                while layout_buffer:
                    plugin_name = weechat.hdata_string(hdata_layout_buffer, layout_buffer, "plugin_name")
                    buffer_name = weechat.hdata_string(hdata_layout_buffer, layout_buffer, "buffer_name")
                    full_name = "{}.{}".format(plugin_name, buffer_name)

                    buffer = weechat.buffer_search("==", full_name)
                    if not buffer:
                        buffer_open_full_name(full_name, noswitch=True)

                    layout_buffer = weechat.hdata_move(hdata_layout_buffer, layout_buffer, 1)
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__" and IMPORT_OK:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        weechat.hook_hsignal("10000|buffer_open_full_name", "buffer_open_full_name_opened_cb", "")
        weechat.hook_hsignal("0|buffer_open_full_name", "buffer_open_full_name_unhandled_cb", "")

        weechat.hook_hsignal("500|buffer_open_full_name", "buffer_open_full_name_table_cb", "")

        weechat.hook_hsignal("500|buffer_open_full_name", "buffer_open_full_name_irc_cb", "")
        weechat.hook_signal("irc_server_connected", "irc_server_connected_cb", "")

        weechat.hook_command(SCRIPT_COMMAND, SCRIPT_DESC,
"""closed [-noswitch|-list]
  || [-noswitch] <full name>""",
"""      closed: open most recently closed buffer
closed -list: list most recently closed buffers
   -noswitch: try not to switch to new buffer

Without subcommand, this command opens a buffer with given full name.

Option "{prefix}.max_closed" specifies the number of most recently closed buffers that are remembered.

If option "{prefix}.layout_apply" is on and "/layout apply" is executed, closed buffers in the layout are opened.""".format(prefix="plugins.var.python.{}".format(SCRIPT_NAME)),
"""closed -noswitch|-list %-
  || -noswitch""".replace("\n", ""),
        "command_cb", "")

        weechat.hook_signal("buffer_closing", "buffer_closing_cb", "")
        weechat.hook_command_run("/layout apply*", "layout_apply_cb", "")

        for option, value in SETTINGS.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, value[0])

            weechat.config_set_desc_plugin(option, "{} (default: \"{}\")".format(value[1], value[0]))



# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Copyright (c) 2015 - 2016 Jos Ahrens <zarthus@lovebytes.me>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# ---
#
# WHOWAS TimeAgo hooks into your WHOWAS calls and adds an easier to read
# timestamp based from the time given by the ircd.
#
# Without this script, you will merely see:
#   [$nick] $server ($timestamp)
# When this script is installed, a new line will display this:
#   [$nick] last seen $time ago
#
# The timestamp sent by the server looks like this:
#   Wed Dec 16 03:06:14 2015
#
# Where time is a short formatted string like so:
#   1 day, 5 hours
#   5 hours, 15 minutes
#   and so on..
#
# Script options:
#   show_errors (default: false, options: true, false)
#     Display script errors (most often caused by unsupported IRCds, and /WHOIS). Asking the user
#     to please report the issue to the maintainer, along with informing how to turn
#     this option off.
#
#     Servers send both numeric 312 on WHOIS and WHOWAS, but they return different things.
#     This often is a cause of an error on WHOIS, and will get noisy very quickly. But this
#     option becomes useful when you're not getting feedback when it was on a WHOWAS.
#
#     Errors are noisy, but easy to fix once known.
#
# This script functions on WeeChat 1.3 and above, supporting both Python 2 and Python 3.
# It probably runs on earlier versions of WeeChat, but has not been tested.
#
# History:
#  version 1.0 - 2015-12-16
#    Script creation
#  version 1.1 - 2015-12-19
#    WHOIS shares the same numeric as WHOWAS uses, and returns different data.
#    This issue has been fixed by defaulting show_errors to false, and displaying extra
#    notices.
#  version 1.2 - 2015-12-19
#    The WHOWAS data is in the UTC timezone, but not everyone is. Ensure we get utctime()

try:
    import weechat as w
    import_ok = True
except ImportError:
    print("This script must be run under WeeChat")
    print("Get WeeChat now at: https://weechat.org/")
    import_ok = False

import datetime
import re


SCRIPT_NAME = "whowas_timeago"
SCRIPT_AUTHOR = "Zarthus <zarthus@lovebytes.me>"
SCRIPT_VERSION = "1.2"
SCRIPT_LICENSE = "MIT"
SCRIPT_DESC = "Display a human-readable time string for WHOWAS data"

PARSE_TIMESTAMP_REGEXP = re.compile("^\w{3} (\w{3}) (\d{1,2}) (\d{2}):(\d{2}):(\d{2}) (\d{4})$")


def whowas_callback_timestamp(data, signal, signal_data):
    """When the WHOWAS date numeric comes by"""

    parsed = w.info_get_hashtable("irc_message_parse", {"message": signal_data})

    server = signal.split(",")[0]
    nick = parsed["arguments"].split(" ")[1]
    timestamp = parsed["arguments"].split(":", 1)[1]
    buff = find_target_buffer(server, nick)

    n = fmt_nick(nick)
    t = fmt_time(timestamp)

    if not t:
        # This was most likely a WHOIS, and contained the server description.
        if w.config_get_plugin("show_errors") in ["true", "yes", "on"]:
            w.prnt(buff, ("error: Failed to parse timestamp '{}' for {}."
                " You can neglect this error if this was a WHOIS request.").format(timestamp, nick))
            w.prnt(buff, "error: Please report this issue to the maintainer of the {} script.".format(SCRIPT_NAME))
            w.prnt(buff, "error: You can turn this notice off by setting `show_errors' to false.")
    else:
        w.prnt(buff, "{} last seen {} ago".format(n, t))

    return w.WEECHAT_RC_OK


def find_target_buffer(server, nick):
    """Return the buffer the user most likely wants their data printed to"""

    targets = {
        "current": w.current_buffer(),
        "weechat": w.buffer_search_main(),
        "server": w.buffer_search("irc", "server.{}".format(server)),
        "private": w.buffer_search("irc", "{}.{}".format(server, nick))
    }

    opt = w.config_string(w.config_get("irc.msgbuffer.whowas"))
    if not opt:
        opt = w.config_string(w.config_get("irc.look.msgbuffer_fallback"))

    target = ""

    if opt.lower() in targets:
        target = targets[opt]

    return target


def parse_timestamp(timestamp):
    """
    Interpret a string that represents time into a manipulatable object

    Timestamp string looks like this:
      Wed Dec 16 03:06:23 2015
    Tested on the following ircds:
      charybdis
      ircd-seven
      inspircd
      hybrid
    """

    m = PARSE_TIMESTAMP_REGEXP.match(timestamp)

    if not m:
        return False

    month_s, day, hours, mins, secs, year = m.groups()
    month = parse_month(month_s)

    return datetime.datetime(int(year), int(month), int(day), int(hours), int(mins), int(secs))


def parse_month(month):
    """Read the string month returned by the ircd and convert it to a number"""

    months = {
      "Jan": 1, "Feb": 2, "Mar": 3, "Apr": 4,
      "May": 5, "Jun": 6, "Jul": 7, "Aug": 8,
      "Sep": 9, "Oct": 10, "Nov": 11, "Dec": 12
    }

    if month not in months:
        if w.config_get_plugin("show_errors") in ["true", "yes", "on"]:
            w.prnt("", "error: Unknown month '{}', probably caused by unsupported ircd.".format(month))
            w.prnt("", "error: Please report this issue to the maintainer of the {} script.".format(SCRIPT_NAME))
            w.prnt("", "error: You can turn this notice off by setting `show_errors' to false.")

        return int(datetime.datetime.utcnow().strftime("%m"))

    return months[month]


def fmt_nick(nick):
    """Format nick in colours for output colouring"""

    green = w.color("green")
    reset = w.color("reset")
    nick_col = w.color(w.info_get("irc_nick_color_name", nick))

    return "{}[{}{}{}]{}".format(green, nick_col, nick, green, reset)


def fmt_time(timestamp):
    """Interpret a timestamp and format it to a time-ago string."""

    then = parse_timestamp(timestamp)
    if not then:
        return False

    diff = datetime.datetime.utcnow() - then
    data = []

    hrdiff = diff.seconds / 3600
    mdiff = diff.seconds % 3600 / 60

    if diff.days != 0:
        data.append("{} day{}".format(diff.days, "s" if diff.days != 1 else  ""))
    if hrdiff != 0:
        data.append("{} hour{}".format(hrdiff, "s" if hrdiff != 1 else ""))
    if mdiff != 0:
        data.append("{} minute{}".format(mdiff, "s" if mdiff != 1 else ""))

    if not data:
         # IRCds will often have forgotten data past a week, so chances are it happened now.
         return str(diff.seconds) + " seconds"
    return ", ".join(data)


if import_ok and w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    settings = {
        "show_errors": ["false", "Display error messages when timestamp cannot be parsed. true or false"]
    }

    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value[0])
            w.config_set_desc_plugin(option, default_value[1])

    w.hook_signal("*,irc_in_312", "whowas_callback_timestamp", "")



# -*- coding: utf-8 -*-
#
# weestats.py, version 0.2 for WeeChat version 0.3
# Latest development version: https://github.com/FiXato/weechat_scripts
#
# Inserts some statistics into your input field about the buffers/windows
#  you have open.
# Example: 151 buffers (46 merged): 135 channels, 9 servers, 3 queries,
#  1 script, 1 python, 1 perl, 1 core; 3 windows
#
## History:
#
### 2012-03-29: FiXato:
# * version 0.1: initial release.
#     * Display a count of all the different buffers you have open.
#     * Display a count of all the open windows.
# * version 0.2: Getting the splits.
#     * Displays the how many vertical and horizontal windows.
#       (not quite sure if my approximation is correct though..)
#     * Fixed possible memleak (forgot to free an infolist)
### 2015-05-02: arza:
# * version 0.3:
#     * handle non-#-channels
#     * numerical sort for buffer info
#     * moved window split info to option -split
#     * simplified the output
### 2019-07-05: Sébastien Helleu:
# * version 0.4:
#     * make script compatible with Python 3
#
## Acknowledgements:
# * Sebastien "Flashcode" Helleu, for developing the kick-ass chat/IRC
#    client WeeChat
#
## TODO:
#   - Add more statistics, such as:
#     - average and total history lines.
#     - average and total topic/title lengths
#     - how many are displayed in a window
#
## Copyright (c) 2012 Filip H.F. "FiXato" Slagter,
#   <FiXato+WeeChat [at] Gmail [dot] com>
#   https://google.com/profiles/FiXato
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

from __future__ import print_function

SCRIPT_NAME     = "weestats"
SCRIPT_AUTHOR   = "Filip H.F. 'FiXato' Slagter <fixato [at] gmail [dot] com>"
SCRIPT_VERSION  = "0.4"
SCRIPT_LICENSE  = "MIT"
SCRIPT_DESC     = "Useless statistics about your open buffers and windows"
SCRIPT_COMMAND  = "weestats"
SCRIPT_CLOSE_CB = "close_cb"

import_ok = True

try:
  import weechat as w
except ImportError:
  print("This script must be run under WeeChat.")
  import_ok = False

def close_cb(*kwargs):
  return w.WEECHAT_RC_OK

def command_main(data, buffer, args):
  infolist = w.infolist_get("buffer", "", "")
  buffer_groups = {}
  results = []
  buffer_count = 0
  merge_count = 0
  numbers = set()
  while w.infolist_next(infolist):
    bplugin = w.infolist_string(infolist, "plugin_name")
    bname = w.infolist_string(infolist, "name")
    bpointer = w.infolist_pointer(infolist, "pointer")
    bnumber = w.infolist_integer(infolist, "number")
    btype = w.buffer_get_string(bpointer, 'localvar_type')
    if not bnumber in numbers:
      numbers.add(bnumber)
    else:
      merge_count += 1

    if btype == 'server':
      bdesc = 'servers'
    elif btype == 'channel':
      bdesc = 'channels'
    elif btype == 'private':
      bdesc = 'queries'
    else:
      bdesc = bplugin

    buffer_groups.setdefault(bdesc,[]).append({'name': bname, 'pointer': bpointer})

  w.infolist_free(infolist)

  infolist = w.infolist_get("window", "", "")
  windows_v = set()
  windows_h = set()
  windows = set()
  while w.infolist_next(infolist):
    window = w.infolist_pointer(infolist, "pointer")
    window_w = w.infolist_integer(infolist, "width_pct")
    window_h = w.infolist_integer(infolist, "height_pct")
    windows.add(window)
    if window_h == 100 and window_w != 100:
      windows_v.add(window)
    elif window_w == 100 and window_h != 100:
      windows_h.add(window)
    #else: #both 100%, thus no splits
  w.infolist_free(infolist)

  window_count = len(windows)

  for desc, buffers in buffer_groups.items():
    buffer_count += len(buffers)
    results.append('%i %s' % (len(buffers), desc))

  buffer_stats = ', '.join(sorted(results, key = lambda item: (int(item.partition(' ')[0]) if item[0].isdigit() else float('inf'), item),reverse=True)) # descending numerical sort of strings
  stats_string = '%i buffers (%i merged): %s; %i windows' % (buffer_count, merge_count, buffer_stats, window_count)
  if '-split' in args:
    stats_string += ": %i vertically / %i horizontally split" % (len(windows_v), len(windows_h))
  w.command("", "/input insert %s" % stats_string)
  return w.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
  if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                      SCRIPT_LICENSE, SCRIPT_DESC, SCRIPT_CLOSE_CB, ""):

    w.hook_command(SCRIPT_COMMAND,
                          SCRIPT_DESC,
                          "-split",
                          "Inserts useless statistics about your open buffers and windows into your input line.\n"
                          "-split: Include information about window splits.",
                          "-split",
                          "command_main", "")
